#lang slideshow
(require slideshow/code
         scheme/class
         scheme/gui/base
         scheme/math
         "utils/utils.rkt"
         "utils/htdp.rkt")

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(slide
 (titlet "How to Design Programs")
 (blank)
 (blank)
 (blank)
 (para #:align 'center (tt "http://www.htdp.org")))

(define (keyword s) (colorize (bt s) "brown"))

(define (here which if-which p)
  (if (eq? which if-which)
      (refocus (htl-append
                gap-size
                (colorize (arrow gap-size 0) "forestgreen")
                p)
               p)
      p))

(define (design-slide which)
  (slide
   #:title "How to Design Programs"
   (here which 'representation
         (item "Determine the" (keyword "representation")))
   (subitem "data definition or" (code define-type))
   (here which 'examples
         (item "Write" (keyword "examples")))
   (subitem (code test))
   (here which 'template
         (item "Create a" (keyword "template") "for the implementation"))
   (vl-append
    (current-line-sep)
    (subitem (code cond) "or" (code type-case) ", if variants")
    (subitem "extract field values, if any")
    (subitem "cross- and self-calls, if data references"))
   (here which 'body
         (item "Finish" (keyword "body") "implementation case-by-case"))
   (here which 'test
         (item "Run" (keyword "tests")))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(design-slide #f)

(define cookies-rep
  (para (code number
              code:blank
              eat-cookie : (number -> number))))

(define posn-rep
  (para (code (define-type Posn
                [posn (x : number) 
                      (y : number)])
              code:blank
              (code:line flip : (Posn -> Posn)))))

(define ant-rep
  (para (code (define-type Ant
                [ant (location : Posn)
                     (weight : number)])
              code:blank
              (code:line ant-at-home? : (Ant -> boolean)))))

(define animal-rep
  (para (code (define-type Animal
                [snake (name : symbol)
                       (weight : number)
                       (food : symbol)]
                [tiger (name : symbol)
                       (stripe-count : number)])
              code:blank
              (code:line heavy-animal? : (Animal -> boolean)))))

(define fish-rep0
 (para (code (define-type ListOfNumber
                [emtpyList]
                [biggerList (n : number)
                            (rest : ListOfNumber)])
             code:blank
             (code:line feed-fish : (ListOfNumber -> ListOfNumber)))))

(define fish-rep
 (para (code (code:comment "A (listof 'a) is either")
             (code:comment " - empty")
             (code:comment " - (cons v r)")
             (code:comment "    where  v : 'a  and  r : (listof 'a)")
             code:blank
             (code:line feed-fish : ((listof number) -> (listof number))))))

(define (wrap-rep p)
  (inset (scale p 0.9)
         0 gap-size 0 0))

(define cookie-examples
  (para
   (code (test (eat-cookie 10) 9)
         (test (eat-cookie 1) 0)
         (test (eat-cookie 0) 0))))

(define posn-examples
  (para
   (code (test (flip (posn 1 17)) (posn 17 1))
         (test (flip (posn -3 4)) (posn 4 -3)))))

(define-syntax-rule (rslide e ...) (slide #:layout 'top e ...))

(rslide
 #:title "Representation"
 (problem "Keep track of the number of cookies in a cookie jar")
 'next
 (wrap-rep cookies-rep))
(rslide
 #:title "Examples"
 cookies-rep
 'next
 (blank)
 cookie-examples)
(rslide
 #:title "Template"
 cookies-rep
 'next
 (blank)
 (para
  (code (define (eat-cookie [n : number])
          ... n ...))))
(rslide
 #:title "Body"
 cookies-rep
 (blank)
 'alts
 (list
  (list
   (para
    (code (define (eat-cookie [n : number])
            ... n ...)))
   'next
   (blank)
   cookie-examples)
  (list
   (para
    (code (define (eat-cookie [n : number])
            (if (> n 0) 
                (- n 1) 
                0))))
   (blank)
   cookie-examples)))

(rslide
 #:title "Representation"
 (problem "Track a position on the screen")
 'next
 (wrap-rep posn-rep))
(rslide
 #:title "Examples"
 posn-rep
 'next
 (blank)
 posn-examples)
(rslide
 #:title "Template"
 posn-rep
 'next
 (blank)
 (para (code (define (flip [p : Posn])
               ... (posn-x p)
               ... (posn-y p) ...)))
 'next
 (t "or")
 (para (code (define (flip [p : Posn])
               (type-case Posn p
                 [posn (x y) ... x ... y ...])))))
(rslide
 #:title "Body"
 posn-rep
 (blank)
 'alts
 (list
  (list
   (para (code (define (flip [p : Posn])
                 (type-case Posn p
                   [posn (x y) ... x ... y ...]))))
   'next
   (blank)
   posn-examples)
  (list
   (para (code (define (flip [p : Posn])
                 (type-case Posn p
                   [posn (x y) (posn y x)]))))
   (blank)
   posn-examples)))
 

(rslide
 #:title "Representation"
 (problem "Track an ant, which has a location and a weight")
 'next
 (wrap-rep ant-rep))
(rslide
 #:title "Examples"
 ant-rep
 'next
 (blank)
 (para
  (code (test 
         (ant-at-home? (ant (posn 0 0) 0.0001)) 
         #t)
        (test 
         (ant-at-home? (ant (posn 5 10) 0.0001)) 
         #f))) )
(rslide
 #:title "Template"
 ant-rep
 'next
 (blank)
 'alts
 (list
  (list
   (para
    (code (define (ant-at-home? [a : Ant])
            (type-case Ant a
              [ant (loc wgt)
                   ... loc ...
                   ... wgt ...])))))
  (list
   (para
    (code (define (ant-at-home? [a : Ant])
            (type-case Ant a
              [ant (loc wgt)
                   ... (is-home? loc) ...
                   ... wgt ...]))))
   'next
   (para
    (code (define (is-home? [p : Posn])
            (type-case Posn p
              [posn (x y) ... x ... y ...])))))))
(rslide
 #:title "Body"
 ant-rep
 (blank)
 'alts
 (list
  (list
   (para
    (code (define (ant-at-home? [a : Ant])
            (type-case Ant a
              [ant (loc wgt)
                   ... (is-home? loc) ...
                   ... wgt ...]))))
   (para
    (code (define (is-home? [p : Posn])
            (type-case Posn p
              [posn (x y) ... x ... y ...])))))
  (list
   (para
    (code (define (ant-at-home? [a : Ant])
            (type-case Ant a
              [ant (loc wgt) (is-home? loc)]))))
   (para
    (code (define (is-home? [p : Posn])
            (type-case Posn p
              [posn (x y) ... x ... y ...])))))
  (list
   (para
    (code (define (ant-at-home? [a : Ant])
            (type-case Ant a
              [ant (loc wgt) (is-home? loc)]))))
   (para
    (code (define (is-home? [p : Posn])
            (type-case Posn p
              [posn (x y) (and (zero? x)
                               (zero? y))])))))))

(rslide
 #:title "Representation"
 (problem "Track an animal, which is a snake or a tiger")
 'next
 (wrap-rep animal-rep))
(rslide
 #:title "Examples"
 animal-rep
 'next
 (blank)
 (para
  (code (test (heavy-animal? (snake 'Slinky 10 'rats)) 
              #t)
        (test (heavy-animal? (snake 'Slimey 8 'cake)) 
              #f)
        (test (heavy-animal? (tiger 'Tony 14)) 
              #t))))
(rslide
 #:title "Template"
 animal-rep
 'next
 (blank)
 (para
  (code (define (heavy-animal? [a : Animal])
          (type-case Animal a
            [snake (n w f)
                   ... n ... w ...
                   ... f ...]
            [tiger (n sc) 
                   ... n ... sc ...])))))
(rslide
 #:title "Body"
 animal-rep
 (blank)
 'alts
 (list
  (list
   (para
    (code (define (heavy-animal? [a : Animal])
            (type-case Animal a
              [snake (n w f)
                     ... n ... w ...
                     ... f ...]
              [tiger (n sc) 
                     ... n ... sc ...])))))
  (list
   (para
    (code (define (heavy-animal? [a : Animal])
            (type-case Animal a
              [snake (n w f) (>= w 10)]
              [tiger (n sc) 
                     ... n ... sc ...])))))
  (list
   (para
    (code (define (heavy-animal? [a : Animal])
            (type-case Animal a
              [snake (n w f) (>= w 10)]
              [tiger (n sc) #t])))))))

 
(rslide
 #:title "Representation"
 (problem "Track an aquarium, which has any number of fish,"
          "each with a weight")
 'next
 'alts
 (list (list (wrap-rep fish-rep0))
       (list (wrap-rep fish-rep))))
(rslide
 #:title "Examples"
 fish-rep
 'next
 (blank)
 (para
  (code (test (feed-fish (list)) (list))
        (test (feed-fish (list 1 2 3)) (list 2 3 4)))))
(rslide
 #:title "Template"
 fish-rep
 'next
 (blank)
 'alts
 (list
  (list
   (para
    (code (define (feed-fish [lon : (listof number)])
            (cond
             [(empty? lon) ...]
             [(cons? lon) ...])))))
  (list
   (para
    (code (define (feed-fish [lon : (listof number)])
            (cond
             [(empty? lon) ...]
             [(cons? lon) 
              ... (first lon) ...
              ... (rest lon) ...])))))
  (list
   (para
    (code (define (feed-fish [lon : (listof number)])
            (cond
             [(empty? lon) ...]
             [(cons? lon) 
              ... (first lon) ...
              ... (feed-fish (rest lon)) ...])))))))
(rslide
 #:title "Body"
 fish-rep
 (blank)
 'alts
 (list
  (list
   (para
    (code (define (feed-fish [lon : (listof number)])
            (cond
             [(empty? lon) ...]
             [(cons? lon) 
              ... (first lon) ...
              ... (feed-fish (rest lon)) ...])))))
  (list
   (para
    (code (define (feed-fish [lon : (listof number)])
            (cond
             [(empty? lon) empty]
             [(cons? lon) 
              ... (first lon) ...
              ... (feed-fish (rest lon)) ...])))))
  (list
   (para
    (code (define (feed-fish [lon : (listof number)])
            (cond
             [(empty? lon) empty]
             [(cons? lon) 
              ... (+ 1 (first lon)) ...
              ... (feed-fish (rest lon)) ...])))))
  (list
   (para
    (code (define (feed-fish [lon : (listof number)])
            (cond
             [(empty? lon) empty]
             [(cons? lon) 
              (cons (+ 1 (first lon))
                    (feed-fish (rest lon)))])))))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Lists

(define list-def
  (code (code:comment "A (listof number) is either")
        (code:comment " - empty")
        (code:comment " - (cons number (listof number))")))

(define lists-title "Implementation Matches Data")

(define list-tmpl
  (code
   code:blank
   (define (feed-fish [lon : (listof number)])
     (cond
      [(empty? lon) ...]
      [(cons? lon) ... (first lon)
       ... (feed-fish (rest lon)) ...]))))

(rslide
 #:title lists-title
 'alts~
 (list
  (list list-def)
  (list
   (add-gp-arrow list-def
                 2/3 25/30 1/2 8/30)))
 'next
 (blank)
 'alts~
 (list
  (list list-tmpl)
  (list
   (add-gp-arrow list-tmpl
                 1/2 55/60 1/3 18/60))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(slide
 (vc-append
  (titlet "How to Design Programs")
  (t "More Examples")))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define show-panels (make-parameter #f))

(define (gui-button s enabled?)
  (let* ([t (inset (bt s) 8)]
         [t (if enabled?
                t
                (colorize t "gray"))])
    (round-frame (cc-superimpose (colorize (filled-rounded-rectangle
                                            (pict-width t)
                                            (pict-height t)
                                            12)
                                           "white")
                                 t)
                 12)))
(define (gui-label s)
  (let ([t (bt s)])
    (inset t 5)))
(define (gui-list l)
  (let ([p (frame (let ([p (inset (apply vl-append 5 (map bt l))
                                  5)])
                    (cc-superimpose (colorize (filled-rectangle (pict-width p)
                                                                (pict-height p))
                                              "white")
                                    p)))])
    (ht-append p (frame (cb-superimpose
                         (ct-superimpose
                          (blank 20 (pict-height p))
                          (frame (arrowhead 20 (/ pi 2))))
                         (frame (arrowhead 20 (/ pi -2))))))))
(define (gui-vertical a b)
  (gui-panel (vc-append 10 a b)))
(define (gui-horizontal a b)
  (gui-panel (hc-append 10 a b)))
(define (gui-panel p)
  (let* ([p (inset p 5)]
         [p (if (show-panels)
                (frame p #:segment 5 #:color "red" #:line-width 2)
                p)])
    (inset p 5)))

(define (gui-frame p)
  (frame (cc-superimpose
          (colorize (filled-rectangle (pict-width p)
                                      (pict-height p))
                    "beige")
          p)))

(define (example-gui)
  (scale
   (gui-frame
    (gui-vertical
     (gui-horizontal
      (gui-label "Pick a fruit:")
      (gui-list '("Apple" "Banana" "Coconut")))
     (gui-horizontal (gui-button "Ok" #f)
                     (gui-button "Cancel" #t))))
   0.8))

(slide
 #:title "GUIs"
 #:layout 'top
 (example-gui)
 (blank)
 (para "Possible programs:")
 (item "Can click?")
 (item "Find a label")
 (item "Read screen"))

(define (gui-slide show-i show-l show-b show-c show-s)
  (slide
   #:title "Representing GUIs"
   #:layout 'top
   (example-gui)
   (blank)
   (parameterize ([current-para-width client-w])
     (rt-superimpose
      (show-i
       (vl-append
        (/ gap-size 2)
        (vc-append
         (current-line-sep)
         (item "labels")
         (show-l (subitem "a label string")))
        (vc-append
         (current-line-sep)
         (item "buttons")
         (show-b (subitem "a label string"))
         (show-b (subitem "enabled state")))
        (vc-append
         (current-line-sep)
         (item "lists")
         (show-c (subitem "a list of choice strings"))
         (show-c (subitem "selected item")))))
      (show-s
       (code (define-type GUI
               [label (text : string)]
               [button (text : string)
                       (enabled? : boolean)]
               [choice (items : (listof string))
                       (selected : number)])))))))
(no-print (gui-slide values ghost ghost ghost ghost))
(no-print (gui-slide values values ghost ghost ghost))
(no-print (gui-slide values values values ghost ghost))
(no-print (gui-slide values values values values ghost))
(gui-slide values values values values values)

(define read-screen-problem
  (problem "Implement" (code read-screen) ", which takes"
           "a GUI and returns a list of strings for all the GUI"
           "element labels"))

(slide
 #:title "Read Screen"
 read-screen-problem)

(slide
 #:title "Read Screen"
 (code
  (define (read-screen [g : GUI]) : (listof string)
    (type-case GUI g
      [label (t) (list t)]
      [button (t e?) (list t)]
      [choice (i s) i]))
  code:blank
  (test (read-screen (label "Hi"))
        (list "Hi"))
  (test (read-screen (button "Ok" true))
        (list "Ok"))
  (test (read-screen (choice (list "Apple" "Banana")
                             0))
        (list "Apple" "Banana"))))

(define define-type:gui
  (scale/improve-new-text
   (code (define-type GUI
           [label (text : string)]
           [button (text : string)
                   (enabled? : boolean)]
           [choice (items : (listof string))
                   (selected : number)]
           [vertical (top : GUI)
                     (bottom : GUI)]
           [horizontal (left : GUI)
                       (right : GUI)]))
   0.8))

(define (gui-panel-slide show-hv show-p show-s show-ex)
  (slide
   #:title "Assemblings GUIs"
   (example-gui)
   (blank)
   (parameterize ([current-para-width client-w])
     (rt-superimpose
      (vl-append
       gap-size
       (item "label")
       (item "buttons")
       (item "lists")
       (show-hv
        (vl-append
         gap-size
         (vc-append
          (current-line-sep)
          (item "vertical stacking")
          (show-p (subitem "two sub-GUIs")))
         (vc-append
          (current-line-sep)
          (item "horizontal stacking")
          (show-p (subitem "two sub-GUIs"))))))
      (show-s define-type:gui)
      (show-ex
       (scale/improve-new-text
        (code (define gui1
                (vertical
                 (horizontal 
                  (label "Pick a fruit:")
                  (choice
                   (list "Apple" "Banana" "Coconut") 
                   0))
                 (horizontal 
                  (button "Ok" false)
                  (button "Cancel" true)))))
        0.9))))))

(no-print (gui-panel-slide ghost ghost ghost ghost))
(parameterize ([show-panels #t])
  (no-print (gui-panel-slide values ghost ghost ghost))
  (no-print (gui-panel-slide values values ghost ghost))
  (gui-panel-slide values values values ghost)
  (gui-panel-slide values values ghost values))

(define read-screen-code
  (scale/improve-new-text
   (code
    (define (read-screen [g : GUI]) : (listof string)
      (type-case GUI g
        [label (t) (list t)]
        [button (t e?) (list t)]
        [choice (i s) i]
        [vertical (t b) (append (read-screen t)
                                (read-screen b))]
        [horizontal (l r) (append (read-screen l)
                                  (read-screen r))])))
   0.8))

(slide
 #:title "Read Screen"
 read-screen-problem)

(slide
 #:title "Read Screen"
 (vl-append
  (current-line-sep)
  read-screen-code
  (scale/improve-new-text
   (code
    code:blank
    ...
    (test (read-screen gui1)
          (list "Pick a fruit:" 
                "Apple" "Banana" "Coconut"
                "Ok" "Cancel")))
   0.8)))

(define (shape-slide add-gp-arrow)
  (slide
   #:title
   "Function and Data Shapes Match"
   (vc-append
    gap-size
    (add-gp-arrow 
     (add-gp-arrow 
      (add-gp-arrow 
       (add-gp-arrow 
        define-type:gui
        0.55 65/100 1/2 1/10)
       0.65 75/100 1/2 1/10)
      0.65 85/100 1/2 1/10)
     0.65 95/100 1/2 1/10)
    (add-gp-arrow 
     (add-gp-arrow 
      (add-gp-arrow 
       (add-gp-arrow 
        read-screen-code
        0.65 55/90 38/100 8/90)
       0.65 65/90 38/100 8/90)
      0.70 75/90 38/100 8/90)
     0.70 85/90 38/100 8/90))))
(no-print (shape-slide (lambda (p x1 y1 x2 y2) p)))
(shape-slide add-gp-arrow)

(slide
 #:title "Design Steps"
 (item "Determine the representation")
 (subitem (code define-type) ", maybe")
 (item "Write examples")
 (subitem (code test))
 (item "Create a template for the implementation")
 (subitem (code type-case) "plus natural recursion,"
               (colorize (bt "check shape!") "red"))
 (item "Finish body implementation case-by-case")
 (subitem (it "usually the interesting part"))
 (item "Run tests"))

(define enable-button-code
  (scale/improve-new-text
   (code
    (define (enable-button [g : GUI] [name : string]) : GUI
      (type-case GUI g
        [label (t) g]
        [button (t e?) (cond
                        [(equal? t name) (button t true)]
                        [else g])]
        [choice (i s) g]
        [vertical (t b) (vertical (enable-button t name)
                                  (enable-button b name))]
        [horizontal (l r) (horizontal (enable-button l name)
                                      (enable-button r name))])))
   0.75))

(slide
 #:title "Enable Button"
 (problem "Implement" (code enable-button)
          ", which takes a GUI and a string"
          "and enables the button whose name matches the string"))

(slide
 #:title "Enable Button"
 (para #:width client-w "The" (code name) "argument is \u201Calong for the ride\u201D:")
 (vl-append
  (current-line-sep)
  enable-button-code
  (scale/improve-new-text
   (code
    ...
    (test (enable-button gui1 "Ok")
          (vertical
           (horizontal (label "Pick a fruit:")
                       (choice (list "Apple" "Banana" "Coconut") 0))
           (horizontal (button "Ok" true)
                       (button "Cancel" true)))))
   0.75)))


(define last-example-title
  "Show Depth")

(parameterize ([show-panels #t])
  (slide
   #:title last-example-title
   (code
    (test (show-depth #,(gui-frame
                         (gui-vertical
                          (gui-label "Hello")
                          (gui-horizontal (gui-button "Ok" #f)
                                          (gui-button "Cancel" #t)))))
          #,(gui-frame
             (gui-vertical
              (gui-label "1 Hello")
              (gui-horizontal (gui-button "2 Ok" #f)
                              (gui-button "2 Cancel" #t))))))))

(slide
 #:title last-example-title
 (para #:width client-w "Template:")
 (code
  (define (show-depth [g : GUI]) : GUI
    (type-case GUI g
      [label (t) ... t ... ]
      [button (t e?) ... t ... e? ...]
      [choice (i s) ... i ... s ...]
      [vertical (t b) ... (show-depth t)
                #||#  ... (show-depth b) ...]
      [horizontal (l r) ... (show-depth l)
                  #||#  ... (show-depth r) ...])))
 'next
 (blank)
 'alts
 (list
  (list
   (hc-append gap-size
              (code (show-depth #,(gui-button "Ok" #t)))
              (t "\u2192")
              (gui-button "0 Ok" #t)))
  (list
   (parameterize ([show-panels #t])
     (let ([dots (t "...")])
       (hc-append gap-size
                  (code (show-depth #,(gui-horizontal (gui-button "Ok" #f)
                                                      (gui-button "Cancel" #t))))
                  (t "\u2192")
                  dots (gui-button "0 Ok" #f) dots (gui-button "0 Cancel" #t) dots))))
  (list
   (para #:align 'right "recursion results don't have the right labels..."))))

(slide
 #:title last-example-title
 (para #:width client-w "The" (code n) "argument is an" (it "accumulator") ":")
 (scale/improve-new-text
  (code
   (define (show-depth-at [g : GUI] [n : number]) : GUI
     (type-case GUI g
       [label (t) (label (prefix n t))]
       [button (t e?) (button (prefix n t) e?)]
       [choice (i s) g]
       [vertical (t b) (vertical (show-depth-at t (+ n 1))
                                 (show-depth-at b (+ n 1)))]
       [horizontal (l r) (horizontal (show-depth-at l (+ n 1))
                                     (show-depth-at r (+ n 1)))]))
   code:blank
   (define (show-depth [g : GUI]) : GUI
     (show-depth-at g 0)))
  0.8))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(slide
 #:title "How to Design Programs"
 (item "Follow the design steps")
 (item "Use accumulators when necessary")
 (item "Reuse functions and/or ``wish'' for helpers"))

