#lang slideshow
(require (lib "code.ss" "slideshow")
         "utils/utils.ss"
         "utils/colors.ss"
         "utils/alg.ss"
         slideshow/play
         slideshow/balloon)

;; ----------------------------------------
(part 1)

(slide
 #:title "Defining A Language's Evaluation"
 (para (tt "lambda-u.rkt") ":")
 (item "With" (tt "#lang plai") sym:implies "an eager language")
 (item "With" (tt "#lang plai-lazy") sym:implies "a lazy language")
 'next
 (blank)
 (para "Let's take control of evaluation order"))

(define (todo p)
  (frame
   (inset
    (vl-append (current-line-sep)
               (scale (t "To do:") 0.8)
               p)
    2)))

(define (plus-todo q p)
  (rtl-superimpose
   (scale
    (todo p)
    0.8)
   q))

(define hole (filled-ellipse 18 18))

(define (align p)
  (para #:width (* client-w 0.95) (scale p 0.8)))

(slide
 #:title "Evaluation and ``To do'' Lists"
 (align (code (interp (addC (numC 1) (numC 2)) mt-env)))
 'next
 (align (code #,sym:implies (num+ (interp (numC 1) mt-env)
                                  (interp (numC 2) mt-env))))
 'next
 (plus-todo (align (code #,sym:implies (interp (numC 1) mt-env)))
            (code (num+ #,hole 
                        (interp (numC 2) mt-env))))
 'next
 (plus-todo (align (code #,sym:implies (numV 1)))
            (code (num+ #,hole 
                        (interp (numC 2) mt-env))))
 'next
 (plus-todo (align (code #,sym:implies (interp (numC 2) mt-env)))
            (code (num+ (numV 1)
                        #,hole)))
 'next
 (plus-todo (align (code #,sym:implies (numV 2)))
            (code (num+ (numV 1)
                        #,hole)))
 'next
 (align (code #,sym:implies (num+ (numV 1) (numV 2)))))

(slide
 #:title "Continuations"
 (para "A ``to do'' list is a" (bit "continuation"))
 'next
 (blank)
 'alts
 (list
  (list
   (todo (code (num+ #,hole 
                     (interp (numC 2) mt-env)))))
  (list
   (todo (code (+ 3
                  (* #,hole
                     (f (rest ls))))))))
 'next
 (blank)
 (para "A" (bit "stack") "is one way to implement continuations")
 (todo
  (code (* #,hole (f (rest ls)))
        (+ 3 #,hole)))
 'next
 (blank)
 (parameterize ([current-font-size (floor (* #e0.8 (current-font-size)))])
   (para #:width (* 0.9 client-w)
         #:align 'right
         "The terms" (bit "stack") "and" (bit "continuation") "are sometimes used interchangably")))

;; ----------------------------------------
(part 2)

(define proto (code doAddK : (Value -> Cont)))

(slide
 #:title "Representing Continuations"
 'alts
 (list
  (list
   (todo (code {+ 3 #,hole}))
   'next
   (blank)
   'alts
   (list
    (list
     (code (define-type Cont
             ....)))
    (list
     (code (define-type Cont
             [doAddK (v : Value)]
             ....))))
   'next
   (blank)
   (code (doAddK (numV 3))))
  (list
   (todo
    (code {+ #,hole {f 0}}))
   'next
   (blank)
   'alts
   (list
    (list
     (code (define-type Cont
             [doAddK (v : Value)]
             ....)))
    (list
     (code (define-type Cont
             [addSecondK (r : ExprC) (e : Env)]
             [doAddK (v : Value)]
             ....))))
   'next
   (blank)
   (code (addSecondK (appC (idC 'f) (numV 0))
                     mt-env)))
  (list
   (todo
    (code {+ #,hole {f 0}}
          {+ 3 #,hole}))
   'next
   (blank)
   'alts
   (list
    (list
     (code (define-type Cont
             [addSecondK (r : ExprC) (e : Env)]
             [doAddK (v : Value)]
             ....)))
    (list
     (code (define-type Cont
             [addSecondK (r : ExprC) (e : Env)
                         (k : Cont)]
             [doAddK (v : Value)
                     (k : Cont)]
             ....)))
    (list
     (code (define-type Cont
             [doneK]
             [addSecondK (r : ExprC) (e : Env)
                         (k : Cont)]
             [doAddK (v : Value)
                     (k : Cont)]
             ....))))
   'next
   (blank)
   (code (addSecondK (appC (idC 'f) (numV 0))
                     mt-env
                     (doAddK (numV 3)
                             (doneK)))))))

(define (make-interp #:next-n [next-n 0]
                     #:k-n [k-n 0]
                     #:add-n [add-n k-n]
                     #:lift-n [lift-n add-n])
  (define k-sep (fade-pict k-n (blank) (tt " ")))
  (define k-arg (fade-pict k-n (blank) (code k)))
  (define env
    (hbl-append (code env)
                k-sep
                k-arg))
  (define env0 (code l env))
  (define env01 (ghost (launder env0)))
  (define env02 (ghost (launder env0)))
  (define Env
    (hbl-append (code Env)
                k-sep
                (fade-pict k-n (blank) (code Cont))))
  (define r (code r))
  (define r1 (ghost (launder r)))
  (define r2 (ghost (launder r)))
  (define env1 (code env))
  (define env11 (ghost (launder env1)))
  (define env12 (hbl-append (ghost (launder env1))
                            k-sep
                            k-arg))
  (define plusK (cellophane (code (addSecondK #,r2 #,env12)) add-n))
  (define plusK1 (ghost (launder plusK)))
  (define plusK2 (ghost (launder plusK)))
  (define env+k (fade-pict next-n
                           env01
                           (code #,env02
                                 #,plusK2)))
  (define l (code (interp #,env+k)))
  (define l1 (ghost (launder l)))
  (define l2 (ghost (launder l)))
  (define l21 (ghost (launder l2)))
  (define l22 (ghost (launder l2)))
  (define add (code (num+ #,(lbl-superimpose l1 
                                             (cellophane hole lift-n))
                          (interp #,r1 #,env11))))
  (define add1 (ghost (launder add)))
  (define add2 (ghost (launder add)))
  (define rhs (fade-pict next-n
                         (fade-pict lift-n
                                    add1
                                    (code #,l21
                                          #,(fade-pict add-n add2 plusK1)))
                         l22))
  (define base
    (code (define interp : (ExprC #,Env -> Value)
            (lambda (a #,env)
              (type-case ExprC a
                ...
                [addC (l r) #,rhs]
                ...)))))
  (let* ([p (slide-pict base
                        (cellophane add (- 1 add-n)) add1 add2
                        lift-n)]
         [p (slide-pict p
                        l2 l21 l22
                        next-n)]
         [p (slide-pict p
                        l l1 l2
                        lift-n)]
         [p (slide-pict p
                        plusK plusK1 plusK2
                        next-n)]
         [p (slide-pict p
                        r r1 r2
                        add-n)]
         [p (slide-pict p
                        env1 env11 env12
                        add-n)]
         [p (slide-pict p
                        env0 env01 env02
                        next-n)])
         
    p))

;; ----------------------------------------
(part 3)

(define using-title (hbl-append (code interp) (titlet " with Continuations")))

(play-n (lambda (n) (make-interp #:lift-n n))
        #:title using-title
        #:skip-last? #t)
(play-n (lambda (n) (make-interp #:lift-n 1 #:add-n n))
        #:title using-title
        #:skip-last? #t)
(play-n (lambda (n) (make-interp #:add-n 1 #:k-n n))
        #:title using-title
        #:skip-last? #t)
(play-n (lambda (n) (make-interp #:k-n 1 #:next-n n))
        #:title using-title)


(define (make-continue #:do-add-n [do-add-n 0]
                       #:add-k-n [add-k-n 0]
                       #:add-n [add-n add-k-n]
                       #:do-n [do-n add-n]
                       #:func-n [func-n do-n]
                       #:cont-n [cont-n func-n])
  (define r (code (numV n)))
  (define r1 (launder (ghost r)))
  (define r2 (launder (ghost r)))
  (slide-pict
   (code (define interp : (ExprC Env Cont -> Value)
           (lambda (a env k)
             (type-case ExprC a
               ...
               [numC (n) #,(fade-pict cont-n
                                      r1
                                      (code (continue k #,r2)))]
              ...)))
         code:blank
         #,(cellophane
            (code
             (define continue : (Cont Value -> Value)
               (lambda (k v)
                 (type-case Cont k
                   ...
                   #,(fade-pict
                      add-n
                      (cellophane (code [doneK () v]) do-n)
                      (fade-pict
                       do-add-n
                       (code
                        [addSecondK (r env next-k)
                                    (interp r env
                                            #,(fade-pict
                                               add-k-n
                                               (code ...)
                                               (code (doAddK v next-k))))])
                       (code
                        [doAddK (v-l next-k)
                                (continue next-k (num+ v-l v))])))
                   ...))))
            func-n))
   r r1 r2
   cont-n))

(play-n (lambda (n) (make-continue #:cont-n n))
        #:title using-title
        #:skip-last? #t)
(play-n (lambda (n) (make-continue #:cont-n 1 #:func-n n))
        #:title using-title
        #:skip-last? #t)
(play-n (lambda (n) (make-continue #:func-n 1 #:do-n n))
        #:title using-title
        #:skip-last? #t)
(play-n (lambda (n) (make-continue #:do-n 1 #:add-n n))
        #:steps 5
        #:title using-title
        #:skip-last? #t)
(play-n (lambda (n) (make-continue #:add-n 1 #:add-k-n n))
        #:title using-title
        #:skip-last? #t)
(play-n (lambda (n) (make-continue #:add-k-n 1 #:do-add-n n))
        #:steps 1
        #:title using-title)

;; ----------------------------------------
(part 4)

(define (interp+continue i c)
  (scale
   (code
    (define (interp [a : ExprC] [env : Env] [k : Cont]) : Value
      (type-case ExprC (code:line a ...)
        #,i
        ...))
    code:blank
    (define (continue [k : Cont] [v : Value]) : Value
      (type-case Cont (code:line k ...)
        #,c
        ...)))
   0.8))

(slide
 #:title using-title
 (interp+continue
  (code [numC (n) (continue k (numV n))])
  (code [doneK () v]))
 'next
 (blank)
 (align (code (interp (numC 5) mt-env (doneK))))
 'next
 (align (code #,sym:implies (continue (doneK) (numV 5))))
 'next
 (align (code #,sym:implies (numV 5))))

(define (simple-steps n l)
  (if ((length l) . <= . n)
      (list (add-between l 'next))
      (append
       (simple-steps n (take l n))
       (simple-steps n (list-tail l (sub1 n))))))

(slide
 #:title using-title
 (interp+continue
  (code [addC (l r) (interp l env (addSecondK r env k))])
  (code [addSecondK (r env next-k)
                    (interp r env (doAddK v next-k))]
        [doAddK (v-l next-k)
                (continue next-k (num+ v-l v))]))
 'next
 (blank)
 'alts
 (simple-steps
  3
  (list
   (align (code (interp (addC (numC 5) (numC 2)) mt-env (doneK))))
   (align (code #,sym:implies (interp (numC 5)
                                      (addSecondK (numC 2) mt-env (doneK)))))
   (align (code #,sym:implies (continue (addSecondK (numC 2) mt-env (doneK))
                                        (numV 5))))
   (align (code #,sym:implies (interp (numC 2) mt-env
                                      (doAddK (numV 5) (doneK)))))
   (align (code #,sym:implies (continue (doAddK (numV 5) (doneK))
                                        (numV 2))))
   (align (code #,sym:implies (continue (doneK)
                                        (numV 7)))))))

(slide
 #:title using-title
 (interp+continue
  (code [lamC (n body)
              (continue k (closV n body env))])
  (blank)))

(slide
 #:title using-title
 (interp+continue
  (code [appC (fun arg) (interp fun env (appArgK arg env k))])
  (code [appArgK (a env next-k)
                 (interp a env (doAppK v next-k))]
        [doAppK (v-f next-k)
                (type-case Value v-f
                  [closV (n body c-env)
                         (interp body (extend-env
                                       (bind n v)
                                       c-env)
                                 next-k)]
                  [else (error ...)])])))

;; ----------------------------------------
(part 5)

(define (aalign p)
  (para #:width (* client-w 0.95) p))

(define (anim-steps #:title title . l)
  (let loop ([l l])
    (define pa (aalign
                (if (procedure? (car l))
                    ((car l) 1)
                    (car l))))
    (define sym (if (eq? (cadr l) '=)
                    (t "=")
                    sym:implies))
    (define next-l (if (eq? (cadr l) '=)
                       (cddr l)
                       (cdr l)))
    (define pb (aalign
                (if (procedure? (car next-l))
                    ((car next-l) 0)
                    (car next-l))))
    (when (procedure? (car l))
      (play-n #:title title
              #:layout 'top
              (lambda (n)
                (aalign ((car l) n)))
              #:skip-last? #t))
    (play-n #:title title
            #:layout 'top
            (lambda (n)
              (vc-append
               pa
               (cellophane (aalign sym) n)
               (cellophane pb n)))
            #:skip-last? (pair? (cdr next-l)))
    (cond
     [(null? (cdr next-l)) (void)]
     [else
      (play-n #:title title
              #:layout 'top
              (lambda (n)
                (define g (aalign pb))
                (define g1 (ghost (launder g)))
                (define g2 (ghost (launder g)))
                (slide-pict (lt-superimpose
                             (vc-append
                              (cellophane pa (- 1 n))
                              (cellophane (aalign sym) (- 1 n))
                              g1)
                             g2)
                            g g1 g2
                            n))
              #:skip-last? #t)
      (loop next-l)])))

(slide
 #:title "Infinite Loop"
 (tt "while (1) { }"))

(slide
 #:title "Space-Bounded Loop"
 (tt "int f() { return f(); }"))

(define book-loop-title "Book-Language Loop")

(slide
 #:title book-loop-title
 (code {let {[f {lambda {f}
                  {f f}}]}
         {f f}})
 'next
 (blank)
 (blank)
 (para #:align 'center "infinite or space-bounded?"))

(anim-steps
 #:title book-loop-title
 (code {let {[f {lambda {f} {f f}}]}
         {f f}})
 (code {{lambda {f} {f f}}
        {lambda {f} {f f}}})
 (code {{lambda {f} {f f}}
        {lambda {f} {f f}}}))

(define (assign p e n)
  (lt-superimpose 
   p
   (cellophane
    (pin-balloon (wrap-balloon e 'ne gap-size 0)
                 (ghost p)
                 p (lambda (p q)
                     (define-values (x y) (lt-find p q))
                     (values x (+ y 14))))
    n)))

(anim-steps
 #:title book-loop-title
 (code (interp #,(parsed {{lambda {f} {f f}}
                          {lambda {f} {f f}}})
               mt-env
               (doneK)))
 (code (interp #,(parsed {lambda {f} {f f}})
               mt-env
               (appArgK #,(parsed {lambda {f} {f f}})
                        mt-env
                        (doneK))))
 (code (continue (appArgK #,(parsed {lambda {f} {f f}})
                          mt-env
                          (doneK))
                 (closV 'f #,(parsed {f f}) mt-env)))
 (code (interp #,(parsed {lambda {f} {f f}})
               mt-env
               (doAppK (closV 'f #,(parsed {f f}) mt-env)
                       (doneK))))
 (code (continue (doAppK (closV 'f #,(parsed {f f}) mt-env)
                         (doneK))
                 (closV 'f #,(parsed {f f}) mt-env)))
 (lambda (n)
   (code (interp #,(parsed {f f})
                 #,(assign (code (extend-env 
                                  (bind 'f (closV 'f #,(parsed {f f}) mt-env))
                                  mt-env))
                           (code E_1)
                           n)
                 (doneK))))
 '=
 (code (interp #,(parsed {f f})
               E_1
               (doneK)))
 (code (interp #,(parsed f)
               E_1
               (appArgK #,(parsed f) E_1
                        (doneK))))
 (code (continue (appArgK #,(parsed f) E_1
                          (doneK))
                 (closV 'f #,(parsed {f f}) mt-env)))
 (code (interp #,(parsed f)
               E_1
               (doAppK (closV 'f #,(parsed {f f}) mt-env)
                       (doneK))))
 (code (continue (doAppK (closV 'f #,(parsed {f f}) mt-env)
                         (doneK))
                 (closV 'f #,(parsed {f f}) mt-env)))
 (code (interp #,(parsed {f f})
               E_1
               (doneK))))

;; ----------------------------------------
(part 6)

(anim-steps
 #:title book-loop-title
 (code {let {[f {lambda {f} {+ 1 {f f}}}]}
         {f f}})
 (code {{lambda {f} {+ 1 {f f}}}
        {lambda {f} {+ 1 {f f}}}})
 (code {+ 1 {{lambda {f} {+ 1 {f f}}}
             {lambda {f} {+ 1 {f f}}}}})
 (code {+ 1 {+ 1 {{lambda {f} {+ 1 {f f}}}
                  {lambda {f} {+ 1 {f f}}}}}}))

(slide
 #:title "Tail Calls"
 (code
  (define (forever x)
    (forever (not x))))
 'next
 (blank)
 (blank)
 (para "Call to" (code forever) "is a" (bit "tail call") ","
       "because there's no work to do after" (code forever) "returns"))

(slide
 #:title "Non-Tail Calls"
 (code
  (define (run-out-of-memory x)
    (not (run-out-of-memory x))))
 'next
 (blank)
 (blank)
 (para "The call to" (code run-out-of-memory) "is" (it "not")
       "a tail call, because there's work to do after it returns"))

(slide
 #:title "Tail Calls"
 (code
  (define (forever x)
    (if (zero? x)
        (forever true)
        (forever false))))
 'next
 (blank)
 (blank)
 (para "Even though the call to" (code forever) "is wrapped in" (code if) ","
       "there's no work to do after" (code forever) "returns")
 'next
 (para "The branches of" (code if) "are in" (bit "tail position")
       "with respect to the" (code if)))

(slide
 #:title "Non-Tail Calls"
 (code
  (define (run-out-of-memory x)
    (if (run-out-of-memory x)
        true
        false)))
 'next
 (blank)
 (blank)
 (para "The call to" (code run-out-of-memory) "is" (it "not")
       "a tail call, because there's work to do after it returns")
 'next
 (para "The test position" (code if) "is" (it "not") "in tail position"
       "with respect to the" (code if)))

(slide
 #:title (hbl-append (code interp) (titlet " and ") (code continue))
 (para "In" (tt "lambda-k.rkt") ":")
 'next
 (item (code interp) "calls" (code continue) "only as a tail call")
 'next
 (item (code continue) "calls" (code interp) "only as a tail call")
 'next
 (item (code lookup) "calls" (code lookup) "only as a tail call")
 'next
 (item "nothing else is recursive")
 'next
 (para "∴ the" (tt "plai-typed") "continuation is always small"))
