#lang slideshow
(require racket/class
         slideshow/code
         slideshow/balloon
         "utils/utils.ss" 
         "utils/alg.ss"
         "utils/colors.ss")

(define fp-title "Functional Programming")

(define WIDE (* client-w 0.9))

(define (java . l)
  (apply vl-append
         (current-line-sep)
         (map tt l)))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(part 1)

(slide
 #:title fp-title
 #:layout 'top
 'next
 (para #:width WIDE
       (bit "Functional programming") "is avoiding state")
 'next
 (blank)
 (code
  (define (area [s : Shape]) : number
    ....)
  code:blank
  (define r (rectangle 10 15))
  (test (area r)
        150)
  (test (area r)
        150)
  ...)
 'next
 (blank)
 (para #:align 'right "The alternative:" (bit "imperative programming")))

(slide
 #:title fp-title
 #:layout 'top
 'next
 (para #:width WIDE
       (bit "Functional programming") "often means using functions as values")
 'next
 (blank)
 (code (map area (list (rectangle 10 5)
                       (square 6)
                       (equilateral-triangle 7))))
 
 'next
 (blank 0 (* gap-size 3))
 (para #:align 'right "The alternative:" (bit "first-order programming") "?"))

(define (make-code id)
  (code
   (define (#,id [s : Shape]) : number
     (type-case Shape s
       [rectangle (w h) ...]
       [square (s) ...]
       [equilateral-triangle (s) ...]))))

(define area-code (make-code (code area)))
(define perimeter-code (make-code (code perimeter)))

(slide
 #:title fp-title
 #:layout 'top
 'next
 (para #:width WIDE
       (bit "Functional programming") "often means" (bit "datatype-oriented programming"))
 'next
 (blank)
 'alts
 (list
  (list
   (code (define-type Shape
           [rectangle (w : number) (h : number)]
           [square (side : number)]
           [equilateral-triangle (side : number)])
         code:blank
         #,area-code))
  (list
   (code
    (define (sum-of-areas [l : (listof Shape)])
      (cond
       [(empty? l) 0]
       [(cons? l) (+ (area (first l))
                     (sum-of-areas (rest l)))])))))
 'next
 (blank 0 (* 2 gap-size))
 (para #:align 'right "The alternative:" (bit "object-oriented programming")))

(define vs-title "Datatype-Oriented versus Object-Oriented")

(define (vs-slide dop oop
                  #:oop-label [oop-label values])
  (slide
   #:title vs-title
   (para #:width client-w "Dataype-oriented: call an operation with a variant")
   (dop
    (scale (hc-append (* 4 gap-size)
                      area-code
                      perimeter-code)
           0.6))
   (blank)
   (oop-label
    (para #:width client-w "Object-oriented: call a variant with an operation"))
   (oop
    (scale (java "class Rectangle extends Shape { ..."
                 "  int area() { ... }"
                 "  int perimeter() { ... }"
                 "}"
                 "class Square extends Shape { ..."
                 "  int area() { ... }"
                 "  int perimeter() { ... }"
                 "}"
                 "class EquilateralTriangle extends Shape { ..."
                 "  int area() { ... }"
                 "  int perimeter() { ... }"
                 "}")
           0.6))))

(define (make-add . l)
  (define p (apply vl-append (current-line-sep) l))
  (lambda (q)
    (rb-superimpose q (hilite #:color "lightblue" (inset p (/ gap-size 2) 0)))))

(define add-dop-note
  (make-add (item #:fill? #f "new" (bt "operation") "⇒" (colorize (t "new") "forestgreen") "function")
            (item #:fill? #f "new" (bt "variant") "⇒ " (colorize (t "change") "red") "functions")))
(define add-oop-note
  (make-add (item #:fill? #f "new" (bt "operation") "⇒" (colorize (t "change") "red") "objects")
            (item #:fill? #f "new" (bt "variant") "⇒ " (colorize (t "new") "forestgreen") "objects")))

(vs-slide values ghost #:oop-label ghost)
(vs-slide values values)
(vs-slide add-dop-note values)
(vs-slide add-dop-note add-oop-note)

;; ----------------------------------------
(part 2)

(slide
 #:title vs-title
 (vc-append
  (current-line-sep)
  (para #:align 'center "Functional programming can be")
  (para #:align 'center "datatype-oriented or object-oriented"))
 'next
 (blank)
 (para #:align 'center "We can use functions to represent objects..."))

(define obj-as-func-title "Representing Objects with Functions")

(define (align . l)
  (apply para #:width (* client-w 0.8) l))

(slide
 #:title obj-as-func-title
 (align
  (scode
   (define-type-alias Shape (-> number))))
 'next
 (align
  (scode
   code:blank
   (define (rectangle w h) : Shape
     (lambda ()
       (* w h)))
   code:blank
   (define (square s) : Shape
     (lambda ()
       (* s s)))
   code:blank
   (define r (rectangle 10 15))
   (r) #,sym:implies 15))
 'next
 (align
  (scode
   code:blank
   (define c (let ([r 10])
               (lambda () (* pi (* r r)))))
   (c) #,sym:implies 314.179)))

;; ----------------------------------------
(part 3)

(slide
 #:title "Objects and Multiple Operations"
 (para #:width WIDE "Simple function implements an object with a single operation:")
 (code
  (define-type-alias Shape (-> number))
  code:blank
  (define r (rectangle 10 15))
  (r))
 'next
 (blank)
 (para #:width WIDE "For multiple operations, could pass a symbol to select:")
 (code
  (define-type-alias Shape (symbol -> number))
  code:blank
  (define r (rectangle 10 15))
  (r 'area)
  (r 'perimeter)))

(slide
 #:title obj-as-func-title
 (align
  (scode
   (define-type-alias Shape (symbol -> number))))
 'next
 (align
  (scode
   (define (rectangle w h) : Shape
     (lambda (op)
       (cond
        [(equal? op 'area) (* w h)]
        [(equal? op 'perimeter) (* 2 (+ w h))])))
   code:blank
   (define (square s) : Shape
     (lambda (op)
       (cond
        [(equal? op 'area) (* s s)]
        [(equal? op 'perimeter) (* 4 s)])))
   code:blank
   (define r (rectangle 10 15))
   (r 'area) #,sym:implies 150
   (r 'perimeter) #,sym:implies 50)))

(slide
 #:title obj-as-func-title
 (align
  (scode
   #,(tt "#lang plai ...")
   code:blank
   (code:comment "A Shape is")
   (code:comment " (hash 'area (-> number)")
   (code:comment "       'bigger-than? (number -> boolean))")
   (code:comment "       ....)")))
 'next
 (align
  (scode
   (define (rectangle w h)
     (hash 'area (lambda () (* w h))
           'bigger-than? (lambda (n) (> (* w h) n))))
   code:blank
   (define (square s)
     (hash 'area (lambda () (* s s))
           'bigger-than? (lambda (n) (> (* s s) n))))
   code:blank
   (define r (rectangle 10 15))
   ((hash-ref r 'area)) #,sym:implies 150
   ((hash-ref r 'bigger-than?) 100) #,sym:implies #t)))

;; ----------------------------------------
(part 4)

(slide
 #:title "Objects without Functions"
 (para "In some contexts:")
 (para #:align 'center (bit "datatype-oriented") "vs." (bit "object-oriented"))
 (para "... choice of organization with implications for extensibility")
 'next
 (blank)
 (para "In other contexts:")
 (para #:align 'center (bit "functional") "vs." (bit "object-oriented"))
 (para "... choice of language primitives"))

(slide
 #:title "Representing Objects with Higher-Order Functions"
 'alts
 (let ([w1 (code w)]
       [w2 (code w)])
   (define p (scode (define (rectangle #,w1 h)
                      (hash 'area (lambda () (* #,w2 h))
                            'bigger-than? (lambda (n) (> (* w h) n))))))
   (list
    (list p)
    (list
     (pin-arrow-line (/ gap-size 2)
                     p
                     w2 ct-find
                     w1 ct-find
                     #:start-angle (* pi 1/4)
                     #:end-angle (* pi -3/4)
                     #:color "orange"
                     #:line-width 3))))
 (blank)
 (para #:width WIDE "Relies on nested functions")
 (parameterize ([current-font-size (floor (* #e0.8 (current-font-size)))])
   (para #:align 'right "... implemented as closures")))

(slide
 #:title "Representing Objects with First-Order Functions"
 (sscode (define-syntax-rule (rectangle init-w init-h)
           (list (hash 'w init-w  'h init-h)
                 (hash 'area (lambda (this)
                               (* (get this w)
                                  (get this h)))
                       'bigger-than? (lambda (this n)
                                       (> (send this area)
                                          n))))))
 'next
 (para #:width WIDE "Could be written as")
 (sscode (define (r-area this)
           (* (get this w) (get this h)))
         code:blank
         (define (r-bigger-than? this n)
           (> (send this area) n))
         code:blank
         (define-syntax-rule (rectangle init-w init-h)
           (list (hash 'w init-w  'h init-h)
                 (hash 'area r-area
                       'bigger-than? r-bigger-than?)))))

(define objs-only-title "Representing Objects with First-Order Functions")

(slide
 #:title objs-only-title
 #:layout 'tall
 (align
  (sscode
   (code:comment "A shape is")
   (code:comment " (list (hash ....)")
   (code:comment "       (hash 'area (shape -> number)")
   (code:comment "             'bigger-than? (shape number -> number)))")))
 'next
 (align
  (sscode
   (define-syntax-rule (get obj-expr field-id)
     (hash-ref (first obj-expr) 'field-id))))
 'next
 (align
  (sscode
   (define-syntax-rule (rectangle init-w init-h)
     (list (hash 'w init-w 'h init-h)
           (hash 'area (lambda (this) 
                         (* (get this w)
                            (get this h)))
                 'bigger-than? (lambda (this arg)
                                 (> (send this area)
                                    arg)))))))
 'next
 (align
  (sscode
   (define-syntax-rule (send obj-expr method-id arg ...)
     (let ([o obj-expr])
       ((hash-ref (second o) 'method-id) o arg ...)))
   code:blank
   (define r (rectangle 10 15))
   (send r bigger-than? 200) #,sym:implies #f)))

(slide
 #:title objs-only-title
 (para #:width WIDE
       "Simplifiction: assume that all methods take one argument, and the argument is named" (code arg))
 (align
  (sscode
   (define-syntax-rule (object ([field-id field-expr] ...)
                               [method-id body-expr] ...)
     ....)))
 'next
 (align
  (sscode
   (define-syntax-rule (rectangle init-w init-h)
     (object ([w init-w]
              [h init-h])
             [area (* (get this w)
                      (get this h))]
             [bigger-than? (> (send this area 0)
                              arg)]))
   code:blank
   (define r (rectangle 10 15))
   (send r area 0)#,sym:implies 150
   (send r bigger-than? 100) #,sym:implies #t)))

;; ----------------------------------------
(part 5)

(slide
 #:title "Functions/Datatypes versus Objects"
 (para (it "So far:"))
 (para #:align 'center "object-oriented interpreter of a functional language")
 'next
 (blank)
 (para (it "Now:"))
 (para #:align 'center "functional interpreter of an object-oriented language"))

(define (encode a b)
  (list (scale
         (vl-append
          gap-size
          (hc-append (tt " ") a)
          (tt "≈")
          (hc-append (tt " ") b))
         0.9)))

(define objs-not-funs "Objects Instead of Functions")

(define field-expr (nonterm "Expr"))
(define method-expr (nonterm "Expr"))

(define object-grammar
  (grammar-table
   (list (nonterm "Expr") eqls (t "...") (blank)
         (blank)         -or- (code {object {#,(repeat (nonterm "Field"))} #,(repeat (nonterm "Method"))}) (blank)
         (blank)         -or- (code {get #,(nonterm "Expr") #,(nonterm "Sym")}) (blank)
         (blank)         -or- (code {send #,(nonterm "Expr") #,(nonterm "Sym") #,(nonterm "Expr")}) (blank)
         (nonterm "Field") eqls (code {#,(nonterm "Sym") #,field-expr}) (blank)
         (nonterm "Method") eqls (code {#,(nonterm "Sym") #,method-expr}) (blank))))

(define (annote p x dir . l)
  (pin-balloon (wrap-balloon (apply para #:fill? #f #:width (* client-w 1/3) l)
                             dir
                             (case dir
                               [(w) (- gap-size)]
                               [else 0])
                             (case dir
                               [(w) 0]
                               [(nw) (- gap-size)]))
               p
               x cc-find))

(slide
 #:title objs-not-funs
 'alts
 (list
  (list
   object-grammar)
  (list
   (annote object-grammar field-expr 'nw "evaluated when object is created"))
  (list
   (annote object-grammar method-expr 'nw "delayed until method is called"))
  (list
   (annote object-grammar method-expr 'nw "method always takes one argument"))
  (list
   (annote object-grammar method-expr 'nw "use" (code arg) "to access method argument"))
  (list
   (annote object-grammar method-expr 'nw "use" (code this) "to access enclosing object"))
  (list
   (annote object-grammar field-expr 'nw (code arg) "and" (code this) "refer to outside object, if any"))))

(slide
 #:title objs-not-funs
 (scale object-grammar 0.9)
 'next
 (blank)
 (blank)
 'alts
 (list
  (list
   (code {object
          {{x 1} {y 2}}}))
  (list
   (code {object
          {{x 1} {y {+ 1 1}}}}))
  (list
   (code {get {object
               {{x 1} {y 2}}}
              x}
         #,sym:implies
         1))
  (list
   (code {object
          {}
          {inc {+ arg 1}}}))
  (list
   (code {send {object
                {}
                {inc {+ arg 1}}}
               inc
               2}
         #,sym:implies
         3))
  (list
   (code {object
          {}
          {inc {+ arg 1}}
          {dec {+ arg -1}}}))
  (list
   (code {object
          {{x 1} {y 2}}
          {mdist {+ {get this x}
                    {get this y}}}}))
  (list
   (code {send {object
                {{x 1} {y 2}}
                {mdist {+ {get this x}
                          {get this y}}}}
               mdist
               0}
         #,sym:implies
         3))))
   
;; ----------------------------------------
(part 6)

(slide
 #:title objs-not-funs
 (para #:width (* 0.9 client-w) "Objects can encode functions:")
 'next
 (blank)
 'alts
 (list
  (encode
   (code {{lambda {x} x}
          5})
   (code {send {object {} {call arg}} 
               call
               5}))
  (encode
   (code {{{lambda {x} {lambda {y} {+ x y}}}
           5}
          6})
   (code
    {send {send {object 
                 {}
                 {call {object
                        {{x arg}}
                        {call {+ arg 
                                 {get this x}}}}}}
                call
                5}
          call
          6}))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(part 7)

(define (ajava  . l)
  (inset
   (let ([p (frame
             (inset
              (scale/improve-new-text 
               (vl-append
                (apply java l))
               0.6)
              (* 2 (current-line-sep))))])
     (refocus (vc-append
               (current-line-sep)
               (scale (t "Analogous Java code") 0.5)
               p)
              p))
   0 (- gap-size) (- (* gap-size 2)) 0))

(define (analogous-to n j)
  (ht-append gap-size n j))

(slide
 #:title "Object Language"
 (scale/improve-new-text
  (grammar-table
   (list (nonterm "Expr") eqls (nonterm "Num") (blank)
         (blank)         -or- (code {+ #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
         (blank)         -or- (code {* #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
         (blank)         -or- (code arg) (blank)
         (blank)         -or- (code this) (blank)
         (blank)         -or- (code {object {#,(repeat (nonterm "Field"))} #,(repeat (nonterm "Method"))}) (blank)
         (blank)         -or- (code {get #,(nonterm "Expr") #,(nonterm "Sym")}) (blank)
         (blank)         -or- (code {send #,(nonterm "Expr") #,(nonterm "Sym") #,(nonterm "Expr")}) (blank)))
  0.8)
 'next
 (blank)
 'alts
 (list (list
        (blank)
        (blank)
        (analogous-to
         (scale
          (sshcode {send {object
                          {{x 3} {y 4}}
                          {mdist {+ {get this x} 
                                    {get this y}}}}
                         call
                         mdist
                         0})
          1.2)
         (ajava "class Posn {"
                "  int x, y;"
                "  int mdist() {"
                "    return this.x + this.y;"
                "  }"  
                "}"
                "new Posn(3,4).mdist()")))
       (list
        (analogous-to
         (sshcode {send {object
                         {{x 1} {y 2} {z 3}}
                         {mdist {+ {get this x} 
                                   {+ {get this y}
                                      {get this z}}}}
                         {addDist {+ {send arg mdist 0}
                                     {send this mdist 0}}}}
                        addDist
                        {object
                         {{x 3} {y 4}}
                         {mdist {+ {get this x} 
                                   {get this y}}}}})
         (ajava "class Posn {"
                "  ... as before ..."
                "}"
                "class Posn3D extends Posn {"
                "  int z; ..."
                "  int mdist() {"
                "    return this.x + this.y + this.z;"
                "  }"
                "  int addDist(Posn p) {"
                "    return p.mdist() + this.mdist();"
                "  }"
                "}"
                "new Posn3D(1,2,3).addDist(new Posn(3,4))")))))

;; ----------------------------------------

(slide
 #:title "Expressions"
 (sscode
  (define-type ExprC
    [numC (n : number)]
    [plusC (lhs : ExprC)
           (rhs : ExprC)]
    [multC (lhs : ExprC)
           (rhs : ExprC)]
    [argC]
    [thisC]
    [objectC (field-names : (listof symbol))
             (field-exprs : (listof ExprC))
             (methods : (listof Method))]
    [getC (obj-expr : ExprC)
          (field-name : symbol)]
    [sendC (obj-expr : ExprC)
           (method-name : symbol)
           (arg-expr : ExprC)])
  code:blank
  (define-type Method
    [method (name : symbol)
            (body-expr : ExprC)])))


(slide
 #:title "Values"
 (scode
  (define-type Value
    [numV (n : number)]
    [objV (field-names : (listof symbol))
          (field-values : (listof Value))
          (methods : (listof Method))])))

(slide
 #:title "Examples"
 'alts
 (list
  (list
   (scode
    interp : (ExprC -> Value)
    code:blank
    (test (interp (plusC (numC 1) (numC 2)))
          (numV 3))))
  (list
   (scode
    (test (interp (objectC empty empty empty))
          (objV empty empty empty))))
  (list
   (scode
    (test (interp (objectC (list 'x)
                           (list (plusC (numC 1)
                                        (numC 2)))
                           (list 
                            (method 'inc (plusC (argC) 
                                                (numC 1))))))
          (objV (list 'x)
                (list (numV 3))
                (list (method 'inc (plusC (argC) 
                                          (numC 1))))))))
  (list
   (scode
    (test (interp (getC
                   (objectC (list 'x)
                            (list (plusC (numC 1) 
                                         (numC 2)))
                            (list 
                             (method 'inc (plusC (argC) 
                                                 (numC 1)))))
                   'x))
          (objV (numV 3)))))
  (list
   (scode
    (test (interp (sendC 
                   (objectC (list 'x)
                            (list (plusC (numC 1) 
                                         (numC 2)))
                            (list
                             (method 'inc (plusC (argC) 
                                                 (numC 1)))))
                   'inc
                   (numC 7)))
          (objV (numV 8)))))
  (list
   (scode
    (test (interp (plusC (argC) (numC 1)))
          ???))
   'next
   (blank)
   (para "Need" (code arg) "and" (code this) "values...")
   'next
   (para "Instead of an environment, just provide 2 values to" (code interp)))
  (list
   (scode
    (define interp : (ExprC Value Value -> Value)
      (lambda (a this-val arg-val)
        ...))
    code:blank
    (test (interp (plusC (argC) (numC 1))
                  (objV empty empty empty)
                  (numV 7))
          (numV 8))))
  (list
   (scode
    (test (interp (getC (thisC) 'x)
                  (objV (list 'x)
                        (list (numV 9))
                        empty)
                  (numV 7))
          (numV 9))))
  (list
   (scode
    (test (interp (plusC (numC 1) (numC 2))
                  (numV -1)
                  (numV -1))
          (numV 3))))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(part 8)

(define (interp-slide keep? middle)
  (if (or keep? (not condense?))
      (slide
       #:title "Interpreter"
       #:layout 'top
       (para
        #:width client-w
        (sscode
         (define interp : (ExprC Value Value -> Value)
           (lambda (a this-val arg-val)
             (type-case ExprC a
               ...
               #,(un-ss-scale middle)
               ...))))))
      (skip-slides 1)))

(interp-slide
 #f
 (blank))

(interp-slide
 #t
 (sscode
  [numC (n) (numV n)]
  [plusC (l r) (num+ (interp l this-val arg-val) 
                     (interp r this-val arg-val))]
  [multC (l r) (num* (interp l this-val arg-val) 
                     (interp r this-val arg-val))]
  [thisC () this-val]
  [argC () arg-val]))

;; ---

(interp-slide
 #f
 (sscode
  [objectC (field-names field-exprs methods)
           ...]))

(interp-slide
 #f
 (sscode
  [objectC (field-names field-exprs methods)
           ... (map (lambda (e)
                      (interp e this-val arg-val))
                    field-exprs)
           ...]))

(interp-slide
 #t
 (sscode
  [objectC (field-names field-exprs methods)
           (objV field-names
                 (map (lambda (e)
                        (interp e this-val arg-val))
                      field-exprs)
                 methods)]))

;; ---

(interp-slide
 #f
 (sscode
  [getC (obj-expr field-name)
        ...]))

(interp-slide
 #f
 (sscode
  [getC (obj-expr field-name)
        ... (interp obj-expr this-val arg-val) ...]))

(interp-slide
 #f
 (sscode
  [getC (obj-expr field-name)
        (type-case Value (interp obj-expr this-val arg-val)
          [objV (field-names field-vals methods)
                ...]
          [else (error 'interp "not an object")])]))

(interp-slide
 #t
 (sscode
  [getC (obj-expr field-name)
        (type-case Value (interp obj-expr this-val arg-val)
          [objV (field-names field-vals methods)
                (get-field field-name field-names field-vals)]
          [else (error 'interp "not an object")])]))

;; ---

(interp-slide
 #f
 (sscode
  [sendC (obj-expr method-name arg-expr)
         ...]))

(interp-slide
 #f
 (sscode
  [sendC (obj-expr method-name arg-expr)
         (local [(define obj
                   (interp obj-expr this-val arg-val))
                 (define arg-val
                   (interp arg-expr this-val arg-val))]
           ...)]))

(interp-slide
 #f
 (sscode
  [sendC (obj-expr method-name arg-expr)
         (local [(define obj
                   (interp obj-expr this-val arg-val))
                 (define arg-val
                   (interp arg-expr this-val arg-val))]
           (type-case Value obj
             [objV (field-names field-vals methods)
                   ...]
             [else (error 'interp "not an object")]))]))

(interp-slide
 #f
 (sscode
  [sendC (obj-expr method-name arg-expr)
         (local [(define obj
                   (interp obj-expr this-val arg-val))
                 (define arg-val
                   (interp arg-expr this-val arg-val))]
           (type-case Value obj
             [objV (field-names field-vals methods)
                   ... (find-method method-name methods)
                   ...]
             [else (error 'interp "not an object")]))]))

(interp-slide
 #f
 (sscode
  [sendC (obj-expr method-name arg-expr)
         (local [(define obj
                   (interp obj-expr this-val arg-val))
                 (define arg-val
                   (interp arg-expr this-val arg-val))]
           (type-case Value obj
             [objV (field-names field-vals methods)
                   (type-case Method (find-method method-name
                                                  methods)
                     [method (name body-expr)
                             ...])]
             [else (error 'interp "not an object")]))]))

(interp-slide
 #t
 (sscode
  [sendC (obj-expr method-name arg-expr)
         (local [(define obj
                   (interp obj-expr this-val arg-val))
                 (define arg-val
                   (interp arg-expr this-val arg-val))]
           (type-case Value obj
             [objV (field-names field-vals methods)
                   (type-case Method (find-method method-name 
                                                  methods)
                     [method (name body-expr)
                             (interp body-expr
                                     obj
                                     arg-val)])]
             [else (error 'interp "not an object")]))]))

;; ----------------------------------------

(slide
 #:title "Helpers"
 (sscode
  (define (make-find [name-of : ('a -> symbol)])
    (lambda (code:line ([name : symbol] [vals : (listof 'a)]) : 'a)
      (cond
       [(empty? vals)
        (error 'find "not found")]
       [else (if (equal? name (name-of (first vals)))
                 (first vals)
                 ((make-find name-of) name (rest vals)))])))
  code:blank
  (define find-method : (symbol (listof Method) -> Method)
    (make-find method-name))))

(slide
 #:title "Helpers"
 (sscode
  (code:comment "A non-list pair:")
  (define-type (Pair 'a 'b)
    [kons (first : 'a) (rest : 'b)])
  code:blank
  code:blank
  (define (get-field [name : symbol] 
                     [field-names : (listof symbol)] 
                     [vals : (listof Value)])
    (code:comment "Pair fields and values, find by field name,")
    (code:comment "then extract value from pair")
    (kons-rest ((make-find kons-first)
                name
                (map2 kons field-names vals))))))
