#lang slideshow
(require racket/class
         slideshow/code
         slideshow/balloon
         "utils/utils.ss" 
         "utils/alg.ss"
         "utils/colors.ss"
         "utils/posn3d-obj.ss")

(define fp-title "Functional Programming")

(define WIDE (* client-w 0.9))

(define (java . l)
  (apply vl-append
         (current-line-sep)
         (map tt l)))

(define dynamic-color "blue")
(define static-color "purple")

(define java/hilite
  (case-lambda
   [(s) (cond
         [(regexp-match #rx"^(.*)_([^_]*)_(.*)$" s)
          => (lambda (m)
               (hbl-append
                (java/hilite (cadr m))
                (colorize (tt (caddr m)) dynamic-color)
                (java/hilite (cadddr m))))]
         [(regexp-match #rx"^(.*)%([^_]*)%(.*)$" s)
          => (lambda (m)
               (hbl-append
                (java/hilite (cadr m))
                (colorize (tt (caddr m)) static-color)
                (java/hilite (cadddr m))))]
         [else
          (java s)])]
   [l (apply
       vl-append
       (current-line-sep)
       (map java/hilite l))]))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(part 1)

(slide
 #:title "Objects and Constructors"
 (scode
  (define (numV n)
    (hash 'apply (lambda (arg-val) ....)
          'number (lambda () ...)))
  code:blank
  (define (closV n body c-env)
    (hash 'apply (lambda (arg-val) ....)
          'number (lambda () ....))))
 'next
 (blank)
 (item "result of" (code numV) "or" (code closV) "is an" (bit "object"))
 'next
 (item "functions for" (code 'apply) "and" (code 'number) "are" (bit "methods"))
 'next
 (item (code n) "," (code body) ", and" (code c-env) "are" (bit "fields"))
 'next
 (item (code numV) "and" (code closV) "themselves are" (bit "constructors"))
 'next
 (para #:align 'right "... and not far from" (bit "classes")))

(slide
 #:title "Classes"
 (para #:width WIDE "Classes play two (dynamic) roles:")
 (item "Object construction")
 (scale (java "class Snake {"
              "  ..."
              "}"
              " "
              "new Snake(\"Slinky\", 10);") 
        0.8)
 'next
 (item "Implementation inheritance")
 (scale
  (java "class Rattlesnake extends Snake {"
        "  ..."
        "}")
  0.8)
 'next
 (subitem "Inheritance of methods")
 (subitem "Static method dispatch"))

(slide
 #:title "Classes: Static and Dynamic Dispatch"
 (ht-append
  gap-size
  (scale/improve-new-text
   (java/hilite
    "class Snake implements Animal {"
    "  ..."
    "  boolean endangers(Animal a) {"
    "    return (_a.slowerThan_(100)"
    "            && _a.isLighter_(this.weight/2));"
    "  }"
    "}"
    " "
    "class Rattlesnake extends Snake {"
    "   ..."
    "   boolean endangers(Animal a) {"
    "     return (!_a.hasThickSkin_()"
    "             || %super.endangers%(a))"
    "   }"
    "}"
    " "
    "Animal a = new Rattlesnake(...);"
    "Animal b = new Mouse(...);"
    " "
    "_a.endangers_(b);")
   0.75)
  (frame
   (inset
    (vc-append
     (* 2 (current-line-sep))
     (colorize (bt "dynamic") dynamic-color)
     (colorize (bt "static") static-color))
    (* 4 (current-line-sep))))))

;; ------------------------------------------------------------
(part 2)

(define (ajava #:scale [s 0.6] . l)
  (inset
   (let ([p (frame
             (inset
              (scale/improve-new-text 
               (vl-append
                (apply java l))
               s)
              (* 2 (current-line-sep))))])
     (refocus (vc-append
               (current-line-sep)
               (scale (t "Analogous Java code") 0.5)
               p)
              p))
   0 (- (* 1/2 gap-size)) (- (* gap-size 2)) 0))

(define (analogous-to n j)
  (ht-append gap-size n j))

(define field-expr (nonterm "Expr"))
(define method-expr (nonterm "Expr"))

(define (stack . l) (apply vl-append 10 l))

(define (iframe p)
  (hilite #:color "beige" p))

(slide
 #:title "Class Language with Explicit Static Calls"
 (scale/improve-new-text
  (hc-append
   (- (* 2 (pict-width new-label)))
   (iframe
    (grammar-table
     (list (nonterm "Class") eqls (code {class #,(nonterm "Sym") 
                                          {#,(repeat (nonterm "Field"))} 
                                          #,(repeat (nonterm "Method"))}) 
           new-label
           (nonterm "Field") eqls (nonterm "Sym") (blank)
           (nonterm "Method") eqls (code {#,(nonterm "Sym") #,method-expr}) (blank))))
   (grammar-table
    (list (nonterm "Expr") eqls (nonterm "Num") (blank)
          (blank)         -or- (code {+ #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
          (blank)         -or- (code {* #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
          (blank)         -or- (code arg) (blank)
          (blank)         -or- (code this) (blank)
          (blank)         -or- (code {new #,(nonterm "Sym") #,(repeat (nonterm "Expr"))}) new-label
          (blank)         -or- (code {get #,(nonterm "Expr") #,(nonterm "Sym")}) (blank)
          (blank)         -or- (code {send #,(nonterm "Expr") #,(nonterm "Sym") #,(nonterm "Expr")}) (blank)
          (blank)         -or- (code {ssend #,(nonterm "Expr") #,(nonterm "Sym") #,(nonterm "Sym") #,(nonterm "Expr")}) new-label)))
  0.7)
 'next
 (blank)
 'alts
 (list (list
        (blank)
        (blank)
        (analogous-to
         (scale
          (stack
           (sshcode {class posn
                      {x y}
                      {mdist {+ {get this x} {get this y}}}
                      {addDist {+ {send arg mdist 0}
                                  {send this mdist 0}}}})
           (sshcode {send {new posn 1 2} 
                          addDist
                          {new posn 3 4}}))
          1.2)
         (ajava "class Posn {"
                "  int x, y;"
                "  int mdist() {"
                "    return this.x + this.y;"
                "  }"
                "  int addDist(Posn p) {"
                "    return p.mdist() + mdist();"
                "  }"
                "}"
                "new Posn(1,2).mdist(new Posn(3,4))")))
       (list
        (analogous-to
         (stack
          (sshcode {class (code:line posn ...)
                     {addDist {+ {send arg mdist 0}
                                 {send this mdist 0}}}})
          (sshcode {class posn3D                    
                     {x y z}
                     {mdist {+ {get this z}
                               {ssend this posn mdist arg}}}
                     {addDist {ssend this posn addDist arg}}})
          (sshcode {send {new posn3D 1 2 3} 
                         addDist
                         {new posn 3 4}}))
         (ajava #:scale 0.55
                "class Posn {"
                "  ... as before ..."
                "}"
                "class Posn3D extends Posn {"
                "  int z; ..."
                "  int mdist() {"
                "    return this.z + super.mdist();"
                "  }"
                "  int addDist(Posn p) {"
                "    return super.addDist(p);"
                "  }"
                "}"
                "new Posn3D(1,2,3).addDist(new Posn(3,4))")))))

;; ----------------------------------------
(part 3)

(slide
 #:title "Object Values"
 (para "How does")
 (code {send {new posn3D 1 2 3} mdist ...})
 (para "dispatch to the right" (code mdist) "?")
 'next
 (blank)
 (para "The result of" (code {new posn3D 1 2 3}) "can now"
       "hold a class tag and field values:")
 (posn3d-obj)
 (para "Look for field names and methods in the class"))

(slide
 #:title "Classes and Object Values"
 (code
  (define-type Value
    [numV (n : number)]
    [objV (class-name : symbol)
          (field-values : (listof Value))])
  code:blank
  (define-type ClassC
    [classC (name : symbol)
            (field-names : (listof symbol))
            (methods : (listof Method))]))
 'next
 (blank)
 (blank)
 (code
  interp : (ExprC (listof ClassC) Value Value 
                  -> Value)))

(define posn-class
  (code
   (define posn-class
     (classC 'posn
             (list 'x 'y)
             (list
              (methodC 'mdist
                       (plusC (getC (thisC) 'x) (getC (thisC) 'y)))
              (methodC 'addDist
                       (plusC (sendC (thisC) 'mdist (numC 0))
                              (sendC (argC) 'mdist (numC 0)))))))))
(define posn3d-class
  (code
   (define posn3D-class
     (classC 'posn3D
             (list 'x 'y 'z)
             (list 
              (methodC 'mdist
                       (plusC (getC (thisC) 'z)
                              (ssendC (thisC) 'posn 'mdist (argC))))
              (methodC 'addDist
                       (ssendC (thisC) 'posn 'addDist (argC))))))))

(define posn27-expr
  (code
   (define posn27
     (newC 'posn (list (numC 2) (numC 7))))))

(define posn531-expr
  (code
   (define posn531
     (newC 'posn3D (list (numC 5) (numC 3) (numC 1))))))

(slide
 #:title "Examples"
 'alts
 (list
  (list
   (code (test (interp (numC 10) 
                       empty
                       (numV -1)
                       (numV -1))
               (numV 10))))
  (list
   (sscode
    #,posn-class
    #,posn3d-class
    code:blank
    (define (interp-posn a)
      (interp a (list posn-class posn3D-class) (numV -1) (numV -1)))))
  (list
   (code (test (interp-posn (newC 'posn 
                                  (list (numC 2)
                                        (numC 7))))
               (objV 'posn
                     (list (numV 2)
                           (numV 7)))))
   'next
   (blank)
   (blank)
   (blank)
   (code #,posn27-expr))
  (list
   (scode #,(hilite
             #:color "lightgray"
             (code #,(scale posn-class 0.75)
                   code:blank
                   #,(scale posn27-expr 0.75)))
          code:blank
          code:blank
          (test (interp-posn (sendC posn27 'mdist (numC 0)))
                (numV 9))))
  (list
   (inset
    (scode #,(hilite
              #:color "lightgray"
              (code #,(scale posn-class 0.75)
                    #,(scale posn3d-class 0.75)
                    #,(scale posn27-expr 0.75)
                    #,(scale posn531-expr 0.75)))
           code:blank
           (test (interp-posn (sendC posn531 'addDist posn27))
                 (numV 18)))
    0 (- (* 2 gap-size)) 0 0))))

;; ----------------------------------------
(part 4)

(define (interp-slide keep? middle)
  (if (or keep? (not condense?))
      (slide
       #:title "Interpreter"
       #:layout 'top
       (para
        #:width client-w
        (sscode
         (define interp : (ExprC (listof ClassC) Value Value -> Value)
           (lambda (a classes this-val arg-val)
             (local [(define (recur expr)
                       (interp expr classes this-val arg-val))]
               (type-case ExprC a
                 ...
                 #,(un-ss-scale middle)
                 ...)))))))
      (skip-slides 1)))

(interp-slide
 #f
 (blank))

;; --------------------

(interp-slide
 #t
 (sscode
  [numC (n) (numV n)]
  [plusC (l r) (num+ (recur l) (recur r))]
  [multC (l r) (num* (recur l) (recur r))]
  [thisC () this-val]
  [argC () arg-val]))


;; --------------------

(interp-slide
 #f
 (sscode
  [newC (class-name field-exprs)
        ...]))

(interp-slide
 #f
 (sscode
  [newC (class-name field-exprs)
        (local [(define c (find-class class-name classes))
                (define vals (map recur field-exprs))]
          ...)]))

(interp-slide
 #f
 (sscode
  [newC (class-name field-exprs)
        (local [(define c (find-class class-name classes))
                (define vals (map recur field-exprs))]
          ...
          (objV class-name vals)
          ...)]))

(interp-slide
 #t
 (sscode
  [newC (class-name field-exprs)
        (local [(define c (find-class class-name classes))
                (define vals (map recur field-exprs))]
          (if (= (length vals) (length (class-field-names c)))
              (objV class-name vals)
              (error 'interp "wrong field count")))]))

;; ----------------------------------------

(slide
 #:title "Helper"
 (sscode
  (define find-class : (symbol (listof ClassC) -> ClassC)
    (make-find class-name))))

;; --------------------

(interp-slide
 #f
 (sscode
  [getC (obj-expr field-name)
        ...]))

(interp-slide
 #f
 (sscode
  [getC (obj-expr field-name)
        (type-case Value (recur obj-expr)
          [objV (class-name field-vals)
                ....]
          [else (error 'interp "not an object")])]))

(interp-slide
 #f
 (sscode
  [getC (obj-expr field-name)
        (type-case Value (recur obj-expr)
          [objV (class-name field-vals)
                (type-case ClassC (find-class class-name 
                                              classes)
                  [classC (name fields methods)
                          ....])]
          [else (error 'interp "not an object")])]))

(interp-slide
 #t
 (sscode
  [getC (obj-expr field-name)
        (type-case Value (recur obj-expr)
          [objV (class-name field-vals)
                (type-case ClassC (find-class class-name 
                                              classes)
                  [classC (name fields methods)
                          (get-field field-name fields 
                                     field-vals)])]
          [else (error 'interp "not an object")])]))

;; --------------------

(interp-slide
 #f
 (sscode
  [sendC (obj-expr method-name arg-expr)
         ...]))

(interp-slide
 #f
 (sscode
  [sendC (obj-expr method-name arg-expr)
         (local [(define obj (recur obj-expr))
                 (define arg-val (recur arg-expr))]
           ...)]))

(interp-slide
 #f
 (sscode
  [sendC (obj-expr method-name arg-expr)
         (local [(define obj (recur obj-expr))
                 (define arg-val (recur arg-expr))]
           (type-case Value obj
             [objV (class-name field-vals)
                   ...]
             [else (error 'interp "not an object")]))]))

(interp-slide
 #t
 (sscode
  [sendC (obj-expr method-name arg-expr)
         (local [(define obj (recur obj-expr))
                 (define arg-val (recur arg-expr))]
           (type-case Value obj
             [objV (class-name field-vals)
                   (call-method class-name method-name classes
                                obj arg-val)]
             [else (error 'interp "not an object")]))]))

;; --------------------

(define (align . l) (apply #:width WIDE para l))

(slide
 #:title "Calling a Method"
 #:layout 'top
 'alts~
 (list
  (list
   (align
    (sscode
     (define (call-method class-name method-name classes
                          obj arg-val)
       ...))))
  (list
   (align
    (sscode
     (define (call-method class-name method-name classes
                          obj arg-val)
       ... (find-class class-name classes) ...))))
  (list
   (align
    (sscode
     (define (call-method class-name method-name classes
                          obj arg-val)
       (type-case ClassC (find-class class-name classes)
         [classC (name field-names methods)
                 ...])))))
  (list
   (align
    (sscode
     (define (call-method class-name method-name classes
                          obj arg-val)
       (type-case ClassC (find-class class-name classes)
         [classC (name field-names methods)
                 ... (find-method method-name 
                                  methods) ...])))))
  (list
   (align
    (sscode
     (define (call-method class-name method-name classes
                          obj arg-val)
       (type-case ClassC (find-class class-name classes)
         [classC (name field-names methods)
                 (type-case MethodC (find-method method-name 
                                                 methods)
                   [methodC (name body-expr)
                            ...])])))))
  (list
   (align
    (sscode
     (define (call-method class-name method-name classes
                          obj arg-val)
       (type-case ClassC (find-class class-name classes)
         [classC (name field-names methods)
                 (type-case MethodC (find-method method-name 
                                                 methods)
                   [methodC (name body-expr)
                            (interp body-expr
                                    classes
                                    obj
                                    arg-val)])])))))))

;; --------------------

(interp-slide
 #f
 (sscode
  [ssendC (obj-expr class-name method-name arg-expr)
          ...]))

(interp-slide
 #f
 (sscode
  [ssendC (obj-expr class-name method-name arg-expr)
          (local [(define obj (recur obj-expr))
                  (define arg-val (recur arg-expr))]
            ...)]))

(interp-slide
 #t
 (sscode
  [ssendC (obj-expr class-name method-name arg-expr)
          (local [(define obj (recur obj-expr))
                  (define arg-val (recur arg-expr))]
            (call-method class-name method-name classes
                         obj arg-val))]))

;; ----------------------------------------
(part 5)

(define c-example
  (sscode {class posn
            {x y}
            {mdist {+ {get this x} {get this y}}}
            {addDist {+ {send arg mdist 0}
                        {send this mdist 0}}}}
          code:blank
          {class posn3D
            {x y z}
            {mdist {+ {get this z} 
                      {ssend this posn mdist arg}}}
            {addDist {ssend this posn addDist arg}}}
          code:blank
          {send {new posn3D 1 2 3} addDist {new posn 3 4}}))

(define i-example
  (sscode {class (code:line posn extends object)
            {x y}
            {mdist {+ {get this x} {get this y}}}
            {addDist {+ {send arg mdist 0}
                        {send this mdist 0}}}}
          code:blank
          {class (code:line posn3D extends posn)
            {z}
            {mdist {+ {get this z} 
                      {super mdist arg}}}}
          code:blank
          {send {new posn3D 1 2 3} addDist {new posn 3 4}}))

(slide
 #:title "Subclasses"
 (para "Subclasses with" (tt "ExprC") ":")
 c-example
 'next
 (blank)
 (vc-append
  (current-line-sep)
  (para "Programmer manually")
  (item "duplicates fields")
  (item "implements method inheritance")))

(slide
 #:title "Subclasses"
 (para (tt "ExprI") "adds" (dt "implementation inheritance") ":")
 i-example)

(slide
 #:title "Class Language with Inheritance"
 (scale/improve-new-text
  (vl-append
   gap-size
   (grammar-table
    (list (nonterm "Class") eqls (code {class (code:line #,(nonterm "Sym") extends #,(nonterm "Sym"))
                                         {#,(repeat (nonterm "Field"))} 
                                         #,(repeat (nonterm "Method"))}) 
          new-label
          (nonterm "Field") eqls (nonterm "Sym") (blank)
          (nonterm "Method") eqls (code {#,(nonterm "Sym") #,method-expr}) (blank)))
   (grammar-table
    (list (nonterm "Expr") eqls (nonterm "Num") (blank)
          (blank)         -or- (code {+ #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
          (blank)         -or- (code {* #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
          (blank)         -or- (code arg) (blank)
          (blank)         -or- (code this) (blank)
          (blank)         -or- (code {new #,(nonterm "Sym") #,(repeat (nonterm "Expr"))}) (blank)
          (blank)         -or- (code {get #,(nonterm "Expr") #,(nonterm "Sym")}) (blank)
          (blank)         -or- (code {send #,(nonterm "Expr") #,(nonterm "Sym") #,(nonterm "Expr")}) (blank)
          (blank)         -or- (code {super #,(nonterm "Sym") #,(nonterm "Expr")}) new-label)))
  0.7))

(slide
 #:title "Compiling Inheritance"
 (hc-append
  gap-size
  (scale i-example 0.65)
  (colorize (arrow gap-size 0) "forestgreen")
  (scale c-example 0.65))
 'next
 (blank)
 (item "merge fields from superclasses")
 'next
 (item "change" (code super) "to" (code ssend))
 'next
 (item "merge/override methods"))

;; ----------------------------------------
(part 6)

(slide
 #:title "Classes"
 (scode
  (define-type ClassI
    [classI (name : symbol)
            (super-name : symbol)
            (field-names : (listof symbol))
            (methods : (listof MethodI))])
  code:blank
  (define-type MethodI
    [methodI (name : symbol)
             (body-expr : ExprI)])))

(slide
 #:title "Expressions"
 (scode
  (define-type ExprI
    [numI (n : number)]
    [plusI (lhs : ExprI)
           (rhs : ExprI)]
    [multI (lhs : ExprI)
           (rhs : ExprI)]
    [argI]
    [thisI]
    [newI (class-name : symbol)
          (args : (listof ExprI))]
    [getI (obj-expr : ExprI)
          (field-name : symbol)]
    [sendI (obj-expr : ExprI)
           (method-name : symbol)
           (arg-expr : ExprI)]
    [superI (method-name : symbol)
            (arg-expr : ExprI)])))

(slide
 #:title "Examples"
 'alts
 (list
  (list
   (code (test (expr-i->c (numI 10))
               (numC 10))))
  (list
   (code (test (expr-i->c (thisI))
               (thisC))))
  (list
   (code (test (expr-i->c (superI 'mdist (numI 0)))
               (ssendC (thisC) #,(colorize (tt "???") "red") 'mdist (numC 0)))))
  (list
   (code expr-i->c : (ExprI symbol -> ExprC)
         code:blank
         (test (expr-i->c (superI 'mdist (numI 0)) 'posn)
               (ssendC (thisC) 'posn 'mdist (numC 0)))))))

(slide
 #:title "Compiling Expressions"
 (code
  (define expr-i->c : (ExprI symbol -> ExprC)
    (lambda (a super-name)
      (local [(define (recur expr)
                (expr-i->c expr super-name))]
        (type-case ExprI a
          [numI (n) (numC n)]
          [plusI (l r) (plusC (recur l) (recur r))]
          [multI (l r) (multC (recur l) (recur r))]
          ...
          [superI (method-name arg-expr)
                  (ssendC (thisC)
                          super-name
                          method-name
                          (recur arg-expr))]))))))
(slide
 #:title "Compiling Methods"
 (code
  (define method-i->c : (MethodI symbol -> MethodC)
    (lambda (m super-name)
      (type-case MethodI m
        [methodI (name body-expr) 
                 (methodC name 
                          (expr-i->c body-expr 
                                     super-name))])))))

(slide
 #:title "Compiling Class Methods"
 (scode
  (define class-i->c-not-flat : (ClassI -> ClassC) 
    (lambda (c)
      (type-case ClassI c
        [classI (name super-name field-names methods)
                (classC name
                        field-names
                        (map (lambda (m)
                               (method-i->c m super-name))
                             methods))])))))

(define (flatten-step inside)
  (list
   (para #:width client-w
         (sscode
          (define flatten-class : (ClassC (listof ClassC) (listof ClassI)
                                          -> ClassC)
            (lambda (c classes i-classes)
              #,inside))))))
  
(define (flatten-super-step inside)
  (list
   (para #:width client-w
         (sscode
          (define flatten-super : (symbol (listof ClassC) (listof ClassI)
                                          -> ClassC)
            (lambda (name classes i-classes)
              #,inside))))))

(slide
 #:title "Flattening a Class"
 'alts
 (list
  (flatten-step
   (code ...))
  (flatten-step
   (code
    (type-case ClassC c
      [classC (name field-names methods)
              ...])))
  (flatten-step
   (code
    (type-case ClassC c
      [classC (name field-names methods)
              ... (flatten-super name 
                                 classes 
                                 i-classes) ...])))
  (flatten-step
   (code
    (type-case ClassC c
      [classC (name field-names methods)
              (type-case ClassC (flatten-super name 
                                               classes 
                                               i-classes)
                [classC (super-name super-field-names super-methods)
                        ...])])))
  (flatten-step
   (code
    (type-case ClassC c
      [classC (name field-names methods)
              (type-case ClassC (flatten-super name 
                                               classes 
                                               i-classes)
                [classC (super-name super-field-names super-methods)
                        (classC name
                                ....
                                ....)])])))
  (flatten-step
   (code
    (type-case ClassC c
      [classC (name field-names methods)
              (type-case ClassC (flatten-super name 
                                               classes 
                                               i-classes)
                [classC (super-name super-field-names super-methods)
                        (classC name
                                (add-fields super-field-names 
                                            field-names)
                                ....)])])))
  (flatten-step
   (code
    (type-case ClassC c
      [classC (name field-names methods)
              (type-case ClassC (flatten-super name 
                                               classes 
                                               i-classes)
                [classC (super-name super-field-names super-methods)
                        (classC name
                                (add-fields super-field-names 
                                            field-names)
                                (add/replace-methods super-methods 
                                                     methods))])])))
  ;; ----------------------------------------
  (flatten-super-step
   (code ...))
  (flatten-super-step
   (code
    ... (find-i-class name i-classes) ...))
  (flatten-super-step
   (code
    (type-case ClassI (find-i-class name i-classes)
      [classI (name super-name field-names i-methods)
              ...])))
  (flatten-super-step
   (code
    (type-case ClassI (find-i-class name i-classes)
      [classI (name super-name field-names i-methods)
              ... (find-class super-name classes) ...])))
  (flatten-super-step
   (code
    (type-case ClassI (find-i-class name i-classes)
      [classI (name super-name field-names i-methods)
              (if (equal? super-name 'object)
                  (classC 'object empty empty)
                  ... (find-class super-name classes) ...)])))
  (flatten-super-step
   (code
    (type-case ClassI (find-i-class name i-classes)
      [classI (name super-name field-names i-methods)
              (if (equal? super-name 'object)
                  (classC 'object empty empty)
                  (flatten-class (find-class super-name classes)
                                 classes
                                 i-classes))])))))


(slide
 #:title "Interpreter"
 (sscode
  (define interp-i : (ExprI (listof ClassI) -> Value)
    (lambda (i-a i-classes)
      (local [(define a (expr-i->c i-a 'object))
              (define classes-not-flat
                (map class-i->c-not-flat i-classes))
              (define classes
                (map (lambda (c)
                       (flatten-class c classes-not-flat i-classes))
                     classes-not-flat))]
        (interp a classes (numV -1) (numV -1)))))))
