#lang slideshow
(require scheme/class
         scheme/gui/base
         slideshow/code
         "utils/utils.ss" 
         "utils/alg.ss"
         "utils/colors.ss"
         "utils/posn3d-obj.ss")

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(part 1)

(slide
 #:title "Classes"
 (sscode {class (code:line posn extends object)
           {x y}
           {mdist {+ {get this x} {get this y}}}
           {addDist {+ {send arg mdist 0}
                       {send this mdist 0}}}}
         code:blank
         {class (code:line posn3D extends posn)
           {z}
           {mdist {+ {get this z} 
                     {super mdist arg}}}}
         code:blank
         {send {new posn3D 1 2 3} addDist {new posn 3 4}}))

(slide
 #:title "Typechecking Programs with Classes"
 (para "A well-formed program should never error with")
 'next
 (item "not a number")
 'alts
 (list (list (code {+ 1 {new posn 1 2}}))
       null)
 (item "not an object")
 'alts
 (list (list (code {send 1 mdist 0}))
       (list (code {get 1 x}))
       null)
 (item "wrong field count")
 'alts
 (list (list (code {new posn3D 1 2}))
       null)
 (vl-append
  (current-line-sep)
  (item "not found")
  (subitem "class, field, or method"))
 'alts
 (list (list (code {new square-circle}))
       (list (code {get {new posn 1 2} z}))
       (list (code {send {new posn 1 2} area}))
       (list (code {class (code:line circle extends object)
                     {}
                     {area {super area arg}}}))))

(slide
 #:title "Typed Class Language"
 (scale/improve-new-text
  (grammar-table
   (list (nonterm "Class") eqls (code {class (code:line #,(nonterm "Sym") extends #,(nonterm "Sym"))
                                         {#,(repeat (nonterm "Field"))} 
                                         #,(repeat (nonterm "Method"))}) 
         (blank)
         (nonterm "Field") eqls (code [#,(nonterm "Sym") : #,(nonterm "Type")]) new-label
         (nonterm "Method") eqls (code {#,(nonterm "Sym") : #,(nonterm "Type") -> #,(nonterm "Type") #,(nonterm "Expr")}) new-label
         (nonterm "Type") eqls (code num) new-label
         (blank)          -or- (nonterm "Sym") new-label))
  0.85))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(part 2)

(define NO (para #:fill? #f (colorize (bt "No") RedColor) sym:emdash))
(define YES (colorize (bt "Yes") GreenColor))

(define (should-check prog result)
  (list
   (para "Is this program well-formed?")
   prog
   'next
   (blank)
   result))

(slide
 #:title "Typechecking Programs with Classes"
 'alts
 (list
  
  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {send {get this x} mdist 0}
                {send {get this y} mdist 0}}}}
    code:blank
    10)
   (para NO "the" (code x) "and" (code y) "fields are not objects"))
  
  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {get this x} {get this z}}}}
    code:blank
    10)
   (para NO (code posn) "has no" (code z) "field"))
  
  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {get this x} {send this get-y 0}}}}
    code:blank
    10)
   (para NO (code posn) "has no" (code get-y) "method"))
  
  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> posn
             {+ {get this x} {get this y}}}}
    code:blank
    10)
   (para NO "result type for" (code mdist)
         "does not match body type"))
  
  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {get this x} {get this y}}}}
    code:blank
    10)
   (para YES))
  
  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {get this x} {get this y}}}}
    code:blank
    {new posn 12})
   (para NO "wrong number of fields in" (code new)))
  
  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {get this x} {get this y}}}}
    code:blank
    {new posn 12 {new posn 1 2}})
   (para NO "wrong field type for first" (code new)))
  
  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {get this x} {get this y}}}
      {clone : num -> posn
             {new posn {get this x} {get this y}}}}
    code:blank
    {send {new posn 1 2} clone 0})
   (para YES))
  
  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {get this x} {get this y}}}
      {clone : num -> posn
             {new posn {get this x} {get this y}}}}
    code:blank
    {class (code:line posn3D extends posn)
      {[z : num]}
      {mdist : num -> num
             {+ {get this z} {super mdist arg}}}}
    code:blank
    {new posn3D 5 7 3})
   (para YES))
  
  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {get this x} {get this y}}}
      {clone : num -> posn
             {new posn {get this x} {get this y}}}}
    code:blank
    {class (code:line posn3D extends posn)
      {[z : num]}
      {mdist : num -> posn
             {new posn 10 10}}}
    code:blank
    {new posn3D 5 7 3})
   (para NO "override of" (code mdist) "changes result type"))

  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {get this x} {get this y}}}
      {clone : num -> posn
             {new posn {get this x} {get this y}}}}
    code:blank
    {class (code:line posn3D extends posn)
      {[z : num]}
      {mdist : num -> num
             {+ {get this z} {super mdist arg}}}
      {clone : num -> num
             10}}
    code:blank
    {new posn3D 5 7 3})
   (para NO "override of" (code clone) "changes result type"))

  (should-check 
   (sscode
    {class (code:line posn extends object)
      {[x : num] [y : num]}
      {mdist : num -> num
             {+ {get this x} {get this y}}}
      {clone : num -> posn
             {new posn {get this x} {get this y}}}}
    code:blank
    {class (code:line posn3D extends posn)
      {[z : num]}
      {mdist : num -> num
             {+ {get this z} {super mdist arg}}}
      {clone : num -> posn
             {new posn3D {get this x} {get this y}
                  {get this z}}}}
    code:blank
    {new posn3D 5 7 3})
   (para YES sym:emdash "which means that we need subtypes"))))

(slide
 #:title "Typechecking Summary"
 (item "Use class names as type")
 (item "Check for field and method existence")
 (item "Check field, method, and argument types")
 (item "Check fields against" (code new))
 (item "Check consistency of overrides")
 (item "Treat subclasses as subtypes"))
 
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(part 3)

(slide
 #:title "Datatypes"
 (scode
  (define-type ClassT
    [classT (name : symbol)
            (super-name : symbol)
            (fields : (listof FieldT))
            (methods : (listof MethodT))])
  code:blank
  (define-type FieldT
    [fieldT (name : symbol)
            (type : Type)])
  code:blank
  (define-type MethodT
    [methodT (name : symbol)
             (arg-type : Type)
             (result-type : Type)
             (body-expr : ExprI)])))

(slide
 #:title "Datatypes"
 (scode
  (define-type Type
    [numT]
    [objT (class-name : symbol)])))

(slide
 #:title "Type Checking"
 (sscode
  (define (typecheck [a : ExprI] [t-classes : (listof ClassT)]) : Type
    (begin
      (map (lambda (t-class)
             (typecheck-class t-class t-classes))
           t-classes)
      (typecheck-expr a t-classes (numT) (objT 'bad))))))

(slide
 #:title "Type Checking: Classes"
 (sscode
  (define (typecheck-class [t-class : ClassT]
                           [t-classes : (listof ClassT)])
    (type-case ClassT t-class
      [classT (name super-name fields methods)
              (map (lambda (m)
                     (begin
                       (typecheck-method m (objT name) t-classes)
                       (check-override m t-class t-classes)))
                   methods)]))))

(slide
 #:title "Type Checking: Methods"
 (sscode
  (define (typecheck-method [method : MethodT]
                            [this-type : Type]
                            [t-classes : (listof ClassT)])
    (type-case MethodT method
      [methodT (name arg-type result-type body-expr)
               (if (is-subtype? (typecheck-expr body-expr t-classes
                                                arg-type this-type)
                                result-type
                                t-classes)
                   (values)
                   (type-error body-expr
                               (to-string result-type)))]))))

(slide
 #:title "Type Checking: Method Overrides"
 (sscode
  (define (check-override [method : MethodT]
                          [this-class : ClassT]
                          [t-classes : (listof ClassT)])
    (local [(define super-name 
              (classT-super-name this-class))
            (define super-method
              (try
               (code:comment "Look for method in superclass:")
               (find-method-in-tree (methodT-name method)
                                    (find-classT super-name t-classes)
                                    t-classes)
               (code:comment "no such method in superclass:")
               (lambda () method)))]
      (if (and (equal? (methodT-arg-type method)
                       (methodT-arg-type super-method))
               (equal? (methodT-result-type method)
                       (methodT-result-type super-method)))
          (values)
          (error 'typecheck (string-append
                             "bad override of "
                             (to-string (methodT-name method)))))))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(part 4)

(define (typecheck-slide middle keep?
                         #:extra-defn [extra-defn (code ....)])
  (if (or keep? (not condense?))
      (slide
       #:title "Type Checking Expressions"
       #:layout 'top
       (para
        #:width client-w
        (sscode
         (define typecheck-expr : (ExprI (listof ClassT) Type Type -> Type)
           (lambda (expr t-classes arg-type this-type)
             (local [(define (recur expr)
                       (typecheck-expr expr t-classes arg-type this-type))
                     #,extra-defn]
               (type-case ExprI expr
                 ....
                 #,(un-ss-scale middle)
                 ....)))))))
      (skip-slides 1)))

(typecheck-slide
 (sscode
  [numI (n) (numT)]
  ....
  [argI () arg-type]
  [thisI () this-type])
 #t)

(typecheck-slide
 (sscode
  [plusI (l r) (typecheck-nums l r)]
  [multI (l r) (typecheck-nums l r)])
 #:extra-defn
 (code
  (define (typecheck-nums l r)
    (type-case Type (recur l)
      [numT ()
            (type-case Type (recur r)
              [numT () (numT)]
              [else (type-error r "num")])]
      [else (type-error l "num")])))
 #t)

(define placeholder (sscode ...))

(define (typecheck-steps . steps)
  (let loop ([steps (cdr steps)][accum (car steps)])
    (if (null? steps)
        (typecheck-slide (accum (blank)) #t)
        (if condense?
            (loop (cdr steps) (lambda (e) (accum ((car steps) e))))
            (begin
              (typecheck-slide (accum placeholder) #f)
              (loop (cdr steps)
                    (lambda (e) (accum ((car steps) e)))))))))

(typecheck-steps
 (lambda (x)
   (sscode
    [(code:line newI (class-name exprs))
     #,(un-ss-scale x)]))
 (lambda (x)
   (sscode
    (local [(define arg-types (map recur exprs))
            (define field-types
              (get-all-field-types class-name t-classes))]
      #,(un-ss-scale x))))
 (lambda (x)
   (sscode
    (if (and (= (length arg-types) (length field-types))
             (foldl (lambda (b r) (and r b))
                    true
                    (map2 (lambda (t1 t2) 
                            (is-subtype? t1 t2 t-classes))
                          arg-types
                          field-types)))
        #,(un-ss-scale x)
        (type-error expr "field type mismatch"))))
 (lambda (x)
   (sscode (objT class-name))))

(typecheck-steps
 (lambda (x)
   (sscode
    [(code:line getI (obj-expr field-name))
     #,(un-ss-scale x)]))
 (lambda (x)
   (sscode
    (type-case Type (recur obj-expr)
      [(code:line objT (class-name))
       #,(un-ss-scale x)]
      [else (type-error obj-expr "object")])))
 (lambda (x)
   (sscode
    (local [(define t-class
              (find-classT class-name t-classes))
            (define field
              (find-field-in-tree field-name
                                  t-class
                                  t-classes))]
      #,(un-ss-scale x))))
 (lambda (x)
   (sscode
    (type-case FieldT field
      [fieldT (name type) #,(un-ss-scale x)])))
 (lambda (x)
   (sscode type)))

(typecheck-steps
 (lambda (x)
   (sscode
    [sendI (obj-expr method-name arg-expr)
           #,(un-ss-scale x)]))
 (lambda (x)
   (sscode
    (local [(define obj-type (recur obj-expr))
            (define arg-type (recur arg-expr))]
      #,(un-ss-scale x))))
 (lambda (x)
   (sscode
    (type-case Type obj-type
      [objT (class-name)
            #,(un-ss-scale x)]
      [else
       (type-error obj-expr "object")])))
 (lambda (x)
   (sscode
    (typecheck-send class-name method-name
                    arg-expr arg-type
                    t-classes))))

(typecheck-steps
 (lambda (x)
   (sscode
    [superI (method-name arg-expr)
            #,(un-ss-scale x)]))
 (lambda (x)
   (sscode
    (local [(define arg-type (recur arg-expr))
            (define this-class 
              (find-classT (objT-class-name this-type)
                           t-classes))]
      #,(un-ss-scale x))))
 (lambda (x)
   (sscode
    (typecheck-send (classT-super-name this-class)
                    method-name
                    arg-expr arg-type
                    t-classes))))

(slide
 #:title "Type Checker: Sends"
 (sscode
  (define (typecheck-send [class-name : symbol]
                          [method-name : symbol]
                          [arg-expr : ExprI]
                          [arg-type : Type]
                          [t-classes : (listof ClassT)])
    (type-case MethodT (find-method-in-tree
                        method-name
                        (find-classT class-name t-classes)
                        t-classes)
      [methodT (name arg-type-m result-type body-expr)
               (if (is-subtype? arg-type arg-type-m t-classes)
                   result-type
                   (type-error arg-expr (to-string arg-type)))]))))

(slide
 #:title "Type Checker: Subtypes"
 (sscode
  (define (is-subclass? name1 name2 t-classes)
    (cond
     [(equal? name1 name2) true]
     [(equal? name1 'object) false]
     [else
      (type-case ClassT (find-classT name1 t-classes)
        [classT (name super-name fields methods)
                (is-subclass? super-name name2 t-classes)])]))
  code:blank
  (define (is-subtype? t1 t2 t-classes)
    (type-case Type t1
      [objT (name1)
            (type-case Type t2 
              [objT (name2)
                    (is-subclass? name1 name2 t-classes)]
              [else false])]
      [else (equal? t1 t2)]))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(part 5)

(slide
 #:title "Interpreter"
 (sscode
  (define interp-t : (ExprI (listof ClassT) -> Value)
    (lambda (a t-classes)
      (interp-i a
                (map strip-types t-classes))))
  code:blank
  (define strip-types : (ClassT -> ClassI)
    (lambda (t-class)
      (type-case ClassT t-class
        [(code:line classT (name super-name fields methods))
         (classI name 
                 super-name
                 (map fieldT-name fields)
                 (map (lambda (m)
                        (type-case MethodT m
                          [methodT (name arg-type res-type body-expr)
                                   (methodI name body-expr)]))
                      methods))])))))

(define (lang name features example)
  (list
   (frame (inset
           (cc-superimpose
            (blank (pict-height example) 0)
            (vc-append
             gap-size 
             (tt name)
             (apply
              vc-append
              (* 2 (current-line-sep))
              (map (lambda (s) (if (string? s) (t s) s)) features))))
           (* 2 (current-line-sep))))
   example))


(define down-arrow (colorize (arrow gap-size (* pi 3/2))
                             GreenColor))

(define ticae-posn
  (ssscode
   {class (code:line posn extends object)
     {[x : num] [y : num]}
     {mdist : num -> num
            {+ {get this x} {get this y}}}
     {addDist : posn -> num
              {+ {send this mdist 0} {send arg mdist 0}}}}))

(define ticae-posn3d
  (ssscode {class (code:line posn3D extends posn)
             {[z : num]}
             {mdist : num -> num
                    {+ {get this z} {super mdist arg}}}}
           {send {new posn3D 7 5 3} mdist 0}))

(slide
 #:title "Implementing Classes"
 #:layout 'tall
 (table
  2
  (append
   (lang "ClassT"
         '("types"
           "")
         (vl-append
          (current-line-sep)
          ticae-posn
          ticae-posn3d))
   (list down-arrow (blank))
   (lang "ClassI"
         '("inheritance"
           "super")
         (ssscode {class (code:line posn extends object)
                    {x y}
                    {mdist {+ {get this x} {get this y}}}
                    {addDist {+ {send this mdist 0} {send arg mdist 0}}}}
                  {class (code:line posn3D extends posn)
                    {z}
                    {mdist {+ {get this z} {super mdist arg}}}}
                  {send {new posn3D 7 5 3} mdist 0}))
   (list down-arrow (blank))
   (lang "ClassC"
         '("method dispatch"
           "fields")
         (ssscode {class posn
                    {x y}
                    {mdist {+ {get this x} {get this y}}}
                    {addDist {+ {dsend this mdist 0} {dsend arg mdist 0}}}}
                  {class posn3D
                    {x y z}
                    {mdist {+ {get this z} {ssend this posn mdist arg}}}
                    {addDist {+ {dsend this mdist 0} {dsend arg mdist 0}}}}
                  {dsend {new posn3D 7 5 3} mdist 0})))
  (cons cc-superimpose lc-superimpose) cc-superimpose
  gap-size (* 0.5 gap-size)))
