#lang slideshow
(require slideshow/code
         "utils/utils.rkt")

(define arrow (let ([p (t "→")])
                (lbl-superimpose p (inset p (/ (pict-width p) 3) 0 0 0))))

(define (questionable p)
  (refocus (hc-append
            (* 2 gap-size)
            p
            (scale (colorize (bt "?") "blue") 2))
           p))

;; ----------------------------------------

(slide (titlet "Part 1"))

(slide
 #:title "Language of Arithmetic"
 'alts
 (list
  (list
   (code {+ 2 1})
   'next
   (code #,arrow 3))
  (list
   (code {* 2 1})
   'next
   (code #,arrow 2))
  (list
   (code {+ 2 {* 4 3}})
   'next
   (code #,arrow 14))
  (list
   (code 2)
   'next
   (code #,arrow 2))))

(define expression-inventory
  (list (item "numbers")
        (vc-append
         (current-line-sep)
         (item "addition expressions")
         (subitem "first and second arguments are expressions"))
        (vc-append
         (current-line-sep)
         (item "multiplication expressions")
         (subitem "first and second arguments are expressions"))))

(slide
 #:title "Representing Expressions"
 'alts
 (list
  expression-inventory
  (list
   (code
    (define-type ArithC
      [numC (n : number)]
      [plusC (l : ArithC) (r : ArithC)]
      [multC (l : ArithC) (r : ArithC)])))))

;; ----------------------------------------

(slide (titlet "Part 2"))

(slide
 #:title "Concrete vs. Abstract Syntax"
 (code {+ 2 1})
 'next
 (blank)
 (code (plusC (numC 2) (numC 1))))
 
(slide
 #:title "Concrete Syntax as an S-expression"
 (code '{+ 2 1})
 'next
 (blank)
 (code (test (parse '{+ 2 1})
             (plusC (numC 2) (numC 1)))))

(slide
 #:title "Concrete Syntax as an S-expression"
 (code
  (code:comment "An arith-S-exp is either")
  (code:comment " - number")
  (code:comment " - (list '+ arith-S-expr arith-S-expr)")
  (code:comment " - (list '* arith-S-expr arith-S-expr)")))

;; ----------------------------------------

(slide (titlet "Part 3"))

(slide
 #:title "Subtraction"
 'alts
 (list
  (list
   (code {- 5 2})
   'next
   (code #,arrow 3))
  (list
   (questionable
    (code
     (define-type ArithC
       [numC (n : number)]
       [plusC (l : ArithC) (r : ArithC)]
       [bminusC (l : ArithC) (r : ArithC)] #,new-label
       [multC (l : ArithC) (r : ArithC)]))))
  (list
   (code {- 5 2})
   'next
   (code = {+ 5 -2}))
  (list
   (code {- 5 2})
   'next
   (code = {+ 5 {* -1 2}}))))

(slide
 #:title "Desugaring"
 (code
  (define-type ArithC
    [numC (n : number)]
    [plusC (l : ArithC) (r : ArithC)]
    [multC (l : ArithC) (r : ArithC)])
  code:blank
  (define-type ArithS (code:comment "not ArithC")
    [numS (n : number)]
    [plusS (l : ArithS) (r : ArithS)]
    [bminusS (l : ArithS) (r : ArithS)]
    [multS (l : ArithS) (r : ArithS)])
  code:blank
  desugar : (ArithS -> ArithC)))

;; ----------------------------------------

(slide (titlet "Part 4"))

(define double-example
  (code {define {double x}
          {+ x x}}
        {define {quadruple x}
          {double {double x}}}
        code:blank
        {quadruple 2}))

(slide
 #:title "Functions"
 'alts
 (list
  (list
   double-example
   'next
   (code #,arrow 8))
  (list
   (questionable
    (code {+ {define {double x} {+ x x}}
             1}))
   'next
   (blank)
   (blank)
   (para #:fill? #f "No: a function" (bit "definition") "is not an expression"))
  (list
   (questionable
    (code {+ {double 4}
             1}))
   'next
   (blank)
   (blank)
   (para #:fill? #f "Yes: a function" (bit "call") "is an expression")
   (blank)
   (blank)
   (parameterize ([current-font-size (floor (* #e0.8 (current-font-size)))])
     (para #:fill? #f
           "We'll use" (bit "call") "and" (bit "application") "interchangably")))))


(slide
 #:title "Function Definitions"
 (code {define {triple x}
         {+ x {+ x x}}})
 'next
 (blank)
 (vl-append
  (current-line-sep)
  (para #:fill? #f "A function has")
  (item #:fill? #f "a name")
  (item #:fill? #f  "an argument name")
  (item #:fill? #f "a" (it "body")))
 'next
 (blank)
 'alts
 (list
  (list
   (questionable
    (code
     (define-type BodyC
       [numC (n : number)]
       [idC (s : symbol)]
       [plusC (l : BodyC) (r : BodyC)]
       [multC (l : BodyC) (r : BodyC)]))))
  (list
   (para "Allow" (code x) "to be an expression, and then")
   (code {+ x {+ x x}})
   (para "is also an expression"))))

(define datatypes
  (code
   (define-type ExprC
     [numC (n : number)]
     [idC (s : symbol)]
     [plusC (l : ExprC) 
            (r : ExprC)]
     [multC (l : ExprC)
            (r : ExprC)]
     [appC (s : symbol)
           (arg : ExprC)])
   code:blank
   (define-type FunDefC
     [fdC (name : symbol) 
          (arg : symbol) 
          (body : ExprC)])))

(slide
 #:title "Functions and Function Calls"
 'alts
 (list
  (append (list*
           (car expression-inventory)
           (item "identifiers")
           (cdr expression-inventory))
          (list
           (vc-append
            (current-line-sep)
            (item "function-call expressions")
            (subitem "a function name and an argument expression"))
           (colorize (hline (/ client-w 2) 0) "forestgreen")
           (vc-append
            (current-line-sep)
            (item "a function definition")
            (subitem "a function name, argument name, and body expression"))))
  (list
   datatypes)))

(define (rightward p)
  (inset p (* client-w 1/3) 0 0 0))

(define (attach-datatype p)
  (let ([p (rightward p)])
    (refocus (lt-superimpose
              (ct-superimpose p
                              (blank client-w 0))
              (frame (inset (scale datatypes 0.7) (/ gap-size 2))))
             p)))

(slide
 #:title "Representing Programs"
 'alts
 (list
  (list (attach-datatype (code {+ 1 2}))
        'next
        (blank)
        (blank)
        (rightward (code (plusC (numC 1) 
                                (numC 2)))))
  (list (attach-datatype (code {+ x 2}))
        'next
        (blank)
        (blank)
        (rightward (code (plusC (idC 'x) 
                                (numC 2)))))
  (list (attach-datatype
         (code {define {plus-two x}
                 {+ x 2}}))
        'next
        (blank)
        (blank)
        (rightward
         (code (fdC 'plus-two
                    'x
                    (plusC (idC 'x)
                           (numC 2))))))
  (list (attach-datatype 
         (code {plus-two 9}))
        'next
        (blank)
        (blank)
        (rightward
         (code (appC 'plus-two 
                     (numC 9)))))
  (list (attach-datatype
         double-example)
        'next
        (blank)
        (rightward
         (scale
          (code 
           (list (fdC 'double 'x 
                      (plusC (idC 'x)
                             (idC 'x)))
                 (fdC 'quadruple 'x
                      (appC 'double 
                            (appC 'double
                                  (idC 'x)))))
           code:blank
           (appC 'quadruple (numC 2)))
          0.9)))))

;; ----------------------------------------

(slide (titlet "Part 5"))

(slide
 #:title "Evaluating Function Calls"
 (para
  (code
   {define {double x}
     {+ x x}}
   code:blank
   {double 3}))
 'next
 (para arrow (code {+ 3 3}))
 (para arrow (code 6))
 'next
 (blank)
 (blank)
 (code interp : (ExprC (listof FunDefC) -> number)
       code:blank
       get-fundef : (symbol (listof FunDefC) -> FunDefC)
       code:blank
       subst : (ExprC symbol ExprC -> ExprC)))
