#lang slideshow
(require "utils/colors.ss"
         "utils/utils.ss"
         "utils/alg.ss"
         slideshow/code
         slideshow/balloon
         racket/math)

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(slide (titlet "Part 1"))

(define (show-eval expr val)
  (hbl-append (* 2 gap-size) expr sym:implies val))


(define old-datatype
  (scode
   (define-type ExprC
     [numC (n : number)]
     [idC (s : symbol)]
     [plusC (l : ExprC) (r : ExprC)]
     [multC (l : ExprC) (r : ExprC)]
     [appC (s : symbol) (arg : ExprC)])))

(define new-datatype
  (scode
   (define-type ExprC
     [numC (n : number)]
     [idC (s : symbol)]
     [plusC (l : ExprC) (r : ExprC)]
     [multC (l : ExprC) (r : ExprC)]
     [appC (s : symbol) (arg : ExprC)]
     [letC (n : symbol) (rhs : ExprC)
           (body : ExprC)])))

(define old-grammar
  (list (nonterm "Expr") eqls (nonterm "Num") (blank)
        (blank)          -or- (code #,(nonterm "Sym")) (blank)
        (blank)          -or- (code {+ #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {* #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {#,(nonterm "Sym") #,(nonterm "Expr")}) (blank)))


(slide
 #:title "Arithmetic+Function Language"
 'alts
 (list
  (list
   (code
    (code:comment "An expr-S-exp is either")
    (code:comment " - number")
    (code:comment " - symbol")
    (code:comment " - (list '+ expr-S-expr expr-S-expr)")
    (code:comment " - (list '* expr-S-expr expr-S-expr)")
    (code:comment " - (list symbol expr-S-expr)"))
   'next
   (blank)
   (grammar-table old-grammar))
  (list
   (grammar-table old-grammar)
   'next
   (blank)
   old-datatype)
  (list
   (grammar-table old-grammar))))

(slide
 #:title "Arithmetic+Function+Local Language"
 (grammar-table
  (append
   old-grammar
   (list (blank)          -or- (code {let {[#,(nonterm "Sym") #,(nonterm "Expr")]} 
                                       #,(nonterm "Expr")}) 
         new-label)))
 'next
 (blank)
 'alts
 (list
  (list (show-eval (code {let {[x {+ 1 2}]}
                           {+ x x}})
                   (code 6)))
  (list (show-eval (code {+ {let {[x {+ 1 2}]}
                              {+ x x}}
                            1})
                   (code 7)))
  (list (show-eval (code {+ {let {[x {+ 1 2}]}
                              {+ x x}}
                            {let {[x {- 4 3}]}
                              {+ x x}}})
                   (code 8)))
  
  (list (show-eval (code {+ {let {[x {+ 1 2}]}
                              {+ x x}}
                            {let {[y {- 4 3}]}
                              {+ y y}}})
                   (code 8)))
  (list (show-eval (code {let {[x {+ 1 2}]}
                           {let {[x {- 4 3}]}
                             {+ x x}}})
                   (code 2)))
  
  (list (show-eval (code {let {[x {+ 1 2}]}
                           {let {[y {- 4 3}]}
                             {+ x x}}})
                   (code 6)))
  
  (list (show-eval (code {let {[x {+ 1 2}]}
                           {let {[x {- 4 x}]}
                             {+ x x}}})
                   (code 2)))
  (list (blank))
  (list new-datatype)))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(slide (titlet "Part 2"))

(slide
 #:title "Substitution"
 (para (code (interp (parse '{let {[x 8]}
                               {+ x x}})
                     ....)))
 'next
 (para (htl-append
        gap-size
        sym:implies
        (code (interp (subst (parse '8)
                             'x
                             (parse '{+ x x}))
                      ....)))))

(define (example prob #:thought [thought #f] ans test)
  (define (add-g l) (if thought (cons ghost l) l))
  (map (lambda (a t th)
         (list (para
                (vl-append
                 (current-line-sep)
                 (hbl-append (scode (code:comment "")) 
                             (if (and thought (eq? th values))
                                 (refocus (pin-balloon 
                                           (wrap-balloon thought 'n 0 (- gap-size))
                                           prob
                                           prob
                                           cb-find)
                                          prob)
                                 prob)
                             (tt " ") (a sym:implies) (tt " ") (a ans))
                 (t test)))))
       (add-g (list ghost values values))
       (add-g (list ghost ghost values))
       (add-g (list values ghost ghost))))

(define subs-local-title "Sustitutions and Local Binding")

(slide
 #:title subs-local-title
 'alts~
 (example (scode (code:line 10 for x  in  {let {[y 17]} x}))
          (scode {let {[y 17]} 10})
          (scode (test (subst (numC 10) 'x (letC 'y (numC 17) (idC 'x)))
                       (letC 'y (numC 17) (numC 10)))))
 'next (blank) 'alts~
 (example (scode (code:line 10 for x  in  {let {[y x]} y}))
          (scode {let {[y 10]} y})
          (scode (test (subst (numC 10) 'x (letC 'y (idC 'x) (idC 'y)))
                       (letC 'y (numC 10) (idC 'y)))))
 'next (blank) 'alts~
 (example (scode (code:line 10 for x  in  {let {[x y]} x}))
          #:thought (scode {let {[x {+ 5 5}]}
                             {let {[x y]}
                               x}})
          (scode {let {[x y]} x})
          (scode (test (subst (numC 10) 'x (letC 'x (idC 'y) (idC 'x)))
                       (letC 'x (idC 'y) (idC 'x)))))
 'next (blank) 'alts~
 (example (scode (code:line 10 for x  in  {let {[x x]} x}))
          #:thought (scode {let {[x {+ 5 5}]}
                             {let {[x x]}
                               x}})
          (scode {let {[x 10]} x})
          (scode (test (subst (numC 10) 'x (letC 'x (idC 'x) (idC 'x)))
                       (letC 'x (numC 10) (idC 'x))))))

(define (subst-code inside)
  (para
   (scale
    (code
     (define (subst [what : ExprC] [for : symbol] [in : ExprC])
       (type-case ExprC in
         ....
         [letC (n rhs body)
               #,inside])))
    0.8)))

(slide
 #:title subs-local-title
 'alts
 (list
  (list (subst-code (code ....)))
  (list (subst-code (code (letC ....))))
  (list (subst-code (code (letC n
                                ....))))
  (list (subst-code (code (letC n
                                (subst what for rhs)
                                ....))))
  (list (subst-code (code (letC n
                                (subst what for rhs)
                                (if (symbol=? n for)
                                    ....
                                    ....)))))
  (list (subst-code (code (letC n
                                (subst what for rhs)
                                (if (symbol=? n for)
                                    body
                                    ....)))))
  (list (subst-code (code
                     (letC n
                           (subst what for rhs)
                           (if (symbol=? n for)
                               body
                               (subst what for body))))))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(slide (titlet "Part 3"))

(define parsing-let-title (hbl-append (titlet "Parsing ") (code let)))

(slide
 #:title parsing-let-title

 (scode
  (code:comment "An expr-S-exp is either ...")
  (code:comment
   "- (list 'let (list (list symbol expr-S-expr))")
  (code:comment
   "         expr-S-expr)"))
 'next
 (blank)
 'alts
 (list
  (list
   (scale
    (code
     (and (s-exp-list? s)
          (= 3 (length (s-exp->list s)))
          (s-exp-symbol? (first (s-exp->list s)))
          (eq? 'let (s-exp->symbol (first (s-exp->list s))))
          (s-exp-list? (second (s-exp->list s)))
          (= 1 (length (s-exp->list (second (s-exp->list s)))))
          (s-exp-list? (first (s-exp->list (second (s-exp->list s)))))
          (= 2 (length (s-exp->list
                        (first (s-exp->list 
                                (second (s-exp->list s)))))))
          (s-exp-symbol? (first (s-exp->list
                                 (first (s-exp->list 
                                         (second (s-exp->list s)))))))))
    0.75))
  (list
   (scale
    (code
     (and (s-exp-list? s)
          (let ([sl (s-exp->list s)])
            (and
             (= 3 (length sl))
             (s-exp-symbol? (first sl))
             (eq? 'let (s-exp->symbol (first sl)))
             (let ([bs (second sl)])
               (and (s-exp-list? bs)
                    (= 1 (length (s-exp->list bs)))
                    (let ([b (first (s-exp->list bs))])
                      (and (s-exp-list? b)
                           (= 2 (length (s-exp->list b)))
                           (s-exp-symbol?
                            (first (s-exp->list b)))))))))))
    0.75))
  (list
   (blank)
   (blank)
   (blank)
   (code
    (s-exp-match? '{let {[SYMBOL ANY]} ANY} s)))))

(slide
 #:title "Parser with S-Expression Matching"
 (scale
  (code
   (define (parse [s : s-expression]) : ExprC
     (cond
      [(s-exp-match? `NUMBER s) (numC (s-exp->number s))]
      [(s-exp-match? `SYMBOL s) (idC (s-exp->symbol s))]
      [(s-exp-match? '{+ ANY ANY} s)
       (plusC (parse (second (s-exp->list s)))
              (parse (third (s-exp->list s))))]
      [(s-exp-match? '{* ANY ANY} s)
       (multC (parse (second (s-exp->list s)))
              (parse (third (s-exp->list s))))]
      [(s-exp-match? '{SYMBOL ANY} s)
       (appC (s-exp->symbol (first (s-exp->list s)))
             (parse (second (s-exp->list s))))]
      [(s-exp-match? '{let {[SYMBOL ANY]} ANY} s)
       (let ([bs (s-exp->list (first
                               (s-exp->list (second
                                             (s-exp->list s)))))])
         (letC (s-exp->symbol (first bs))
               (parse (second bs))
               (parse (third (s-exp->list s)))))]
      [else (error 'parse "invalid input")])))
  0.75))

(slide
 #:title (hbl-append (titlet "Getting ") (code s-exp-match?))
 (item "Install the " (tt "plai-typed-s-exp-match") " package")
 (blank)
 (vc-append
  (current-line-sep)
  (item "Add")
  (code (require plai-typed/s-exp-match))
  (item #:bullet (ghost bullet) "to your program")))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(slide (titlet "Part 4"))

(define (align . args)
  (apply para #:width client-w args))

(slide
 #:title "Cost of Substitution"
 (align
  (code (interp #,(parsed
                   {let {[x 1]}
                     {let {[y 2]}
                       {+ 100 {+ 99 {+ 98 ... {+ y x}}}}}}))))
 'next (align sym:implies)
 (align
  (code (interp #,(parsed
                   {let {[y 2]}
                     {+ 100 {+ 99 {+ 98 ... {+ y 1}}}}}))))
 'next (align sym:implies)
 (align
  (code (interp #,(parsed {+ 100 {+ 99 {+ 98 ... {+ 2 1}}}}))))
 'next
 (blank)
 (blank)
 (colorize
  (para #:align 'center
        "With" (bt "n") "variables, evaluation will take" 
        (hbl-append (t "O(")
                    (bt "n") 
                    (text "2" `(superscript . ,(current-main-font)) (current-font-size))
                    (t ")"))
        "time!")
  BlueColor))

(define sym:emptyset (blank))

(define % (tt ""))

(slide
 #:title "Deferring Substitution"
 (align
  (code (interp #,(parsed+sub
                   {let {[x 1]}
                     {let {[y 2]}
                       {+ 100 {+ 99 {+ 98 ... {+ y x}}}}}}
                   code:blank))))
 'next (align sym:implies)
 (align
  (code (interp #,(parsed+sub
                   {let {[y 2]}
                     {+ 100 {+ 99 {+ 98 ... {+ y x}}}}}
                   (code:line x = 1)))))
 'next (align sym:implies)
 (align
  (code (interp #,(parsed+sub
                   {+ 100 {+ 99 {+ 98 ... {+ y x}}}}
                   (code:line y = 2 #,% x = 1)))))
 'next (align sym:implies (code ...) sym:implies)
 (align
  (code (interp #,(parsed+sub y
                              (code:line y = 2 #,% x = 1))))))


(slide
 #:title "Deferring Substitution with the Same Identifier"
 (align
  (code (interp #,(parsed+sub
                   {let {[x 1]}
                     {let {[x 2]}
                       x}}
                   code:blank))))
 'next (align sym:implies)
 (align
  (code (interp #,(parsed+sub
                   {let {[x 2]}
                     x}
                   (code:line x = 1)))))
 'next (align sym:implies)
 (align
  (code (interp #,(parsed+sub
                   x
                   (code:line x = 2 #,% x = 1)))))
 'next
 (blank)
 (para #:align 'center "Always add to start, then always check from start"))

(slide
 #:title "Representing Deferred Substitution: Environments"
 (para #:align 'center "Change")
 (item #:bullet (ghost bullet)
       (code interp : (ExprC (listof FunDef) -> number)))
 (para #:align 'center"to")
 (item #:bullet (ghost bullet)
       (code interp : (ExprC Env (listof FunDef) -> number)))
 'next
 (blank)
 (code
  mt-env : Env
  extend-env : (Binding Env -> Env)
  bind : (symbol number -> Binding)
  lookup : (symbol Env -> number))
 'next
 (blank)
 (blank)
 'alts
 (list
  (list 
   (ht-append gap-size
              (balloon-pict (balloon-code code:blank))
              (scode mt-env)))
  (list
   (ht-append gap-size
              (balloon-pict (balloon-code x = 1))
              (scode (extend-env (bind 'x 1)
                                 mt-env))))
  (list
   (ht-append gap-size
              (balloon-pict (balloon-code y = 2 #,% x = 1))
              (scode (extend-env (bind 'y 2)
                                 (extend-env (bind 'x 1)
                                             mt-env)))))))

(slide
 #:title "Environments"
 (code
  (define-type Binding
    [bind (name : symbol)
          (val : number)])
  code:blank
  (define-type-alias Env (listof Binding))
  code:blank
  (define mt-env empty)
  (define extend-env cons)))

(slide
 #:title "Environment Lookup"
 (code
  (define (lookup [n : symbol] [env : Env]) : number
    (cond
     [(empty? env) (error 'lookup "free variable")]
     [else (cond
            [(symbol=? n (bind-name (first env)))
             (bind-val (first env))]
            [else (lookup n (rest env))])]))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(slide (titlet "Part 5"))

(define (align+implies x)
  (item #:bullet sym:implies
        #:width client-w
        x))

(define interp-with-env-title "Interp with Environments")

(slide
 #:title interp-with-env-title
 (align
  (scode (interp #,(parsed
                    {let {[x 1]}
                      {let {[y 2]}
                        {+ 100 {+ 99 {+ 98 ... {+ y x}}}}}})
                 mt-env)))
 'next
 (align+implies
  (scode (interp #,(parsed
                    {let {[y 2]}
                      {+ 100 {+ 99 {+ 98 ... {+ y x}}}}})
                 (extend-env (bind 'x 1) mt-env))))
 'next 
 (align+implies
  (scode (interp #,(parsed
                    {+ 100 {+ 99 {+ 98 ... {+ y x}}}})
                 (extend-env (bind 'y 2)
                             (extend-env (bind 'x 1) 
                                         mt-env)))))
 'next
 (inset
  (vc-append
   (current-line-sep)
   (align sym:implies (code ...))
   (align+implies
    (scode (interp #,(parsed y) (extend-env (bind 'y 2)
                                            (extend-env (bind 'x 1) 
                                                        mt-env))))))
  0 (- gap-size) 0 0))

(define (wae-interp with-case id-case [fun-case (code ...)])
  (align
   (scale
    (code
     (define (interp [a : ExprC] [env : Env] [fds : (listof FunDefC)])
       (type-case ExprC a
         [numC (n) n]
         [idC (s) #,id-case]
         [plusC (l r) (+ (interp l env fds) (interp r env fds))]
         [multC (l r) (* (interp l env fds) (interp r env fds))]
         [appC (s arg) (local [(define fd (get-fundef s fds))]
                         #,fun-case)]
         [letC (n rhs body) #,with-case])))
    0.8)))

(define lookup-call (code (lookup s env)))

(define (place view? gen inner)
  (define g (ghost (launder inner)))
  (define-values (norm dots) (gen g))
  (define p (pin-over ((if view? values ghost) norm) g lt-find inner))
  (if view?
      p
      (lt-superimpose p dots)))

(define (with-case #:interp? [interp? #f]
                   #:extend? [extend? interp?]
                   #:bind? [bind? extend?])
  (place
   interp?
   (lambda (x)
     (values
      (code (interp body
                    #,x
                    fds))
      (code ...
            code:blank
            code:blank
            code:blank
            ...)))
   (place
    extend?
    (lambda (x)
      (values
       (code
        (extend-env
         #,x
         env))
       (code
        ...
        code:blank
        ...)))
    (place
     bind?
     (lambda (x)
       (values
        (code (bind n #,x))
        (code ...)))
     (code (interp rhs env fds))))))

(define with-rhs (with-case #:interp? #t))

(define (app-case #:extend? [extend? #f]
                  #:interp? [interp? extend?]
                  #:bind? [bind? interp?])
  (place
   interp?
   (lambda (x)
     (values
      (code (interp (fdC-body fd)
                    #,x
                    fds))
      (code ...
            code:blank
            ...)))
   (place
    extend?
    (lambda (x)
      (values
       (code (extend-env
              #,x
              mt-env))
       (code ...
             code:blank
             ...)))
    (place
     bind?
     (lambda (x)
       (values
        (code (bind (fdC-arg fd)
                    #,x))
        (code ...)))
     (code (interp arg env fds))))))

(slide
 #:title interp-with-env-title
 'alts
 (list (list (wae-interp (code ...) (code ...)))
       (list (wae-interp (code ...)
                         lookup-call))
       (list (wae-interp (with-case)
                         lookup-call))
       (list (wae-interp (with-case #:bind? #t)
                         lookup-call))
       (list (wae-interp (with-case #:extend? #t)
                         lookup-call))
       (list (wae-interp with-rhs
                         lookup-call))
       (list (wae-interp with-rhs
                         lookup-call
                         (app-case)))
       (list (wae-interp with-rhs
                         lookup-call
                         (app-case #:bind? #t)))
       (list (wae-interp with-rhs
                         lookup-call
                         (app-case #:interp? #t)))))

(slide
 #:title "Function Calls"
 (align
  (code {define {bad x} {+ x y}}))
 (blank)
 (align
  (code (interp #,(parsed+sub
                   {let {[y 2]}
                     {bad 10}}
                   code:blank))))
 'next (align sym:implies)
 (align
  (code (interp #,(parsed+sub
                   {bad 10}
                   (code:line y = 2)))))
 'next (align sym:implies)
 'alts
 (list (list
        (align
         (code (interp #,(parsed+sub
                          {+ x y}
                          (code:line ...))))))
       (list
        (align
         (code (interp #,(parsed+sub
                          {+ x y}
                          (code:line x = 10)))))))
 (blank)
 (blank)
 (para #:align 'center
       "Interpreting function body starts with only one substitution"))

(slide
 #:title interp-with-env-title
 'alts
 (list (list (wae-interp with-rhs
                         lookup-call
                         (app-case #:interp? #t)))
       (list (wae-interp with-rhs
                         lookup-call
                         (app-case #:extend? #t)))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(slide (titlet "Part 6"))

(define binding-color "lightblue")
(define bound-color "yellowgreen")
(define free-color "pink")

(define (binding-slide #:free-alt? [free-alt? #f]
                       #:free-hilite? [free-hilite? free-alt?]
                       #:free-example? [free-example? free-hilite?]
                       #:bound-alt? [bound-alt? free-example?]
                       #:bound-hilite? [bound-hilite? bound-alt?]
                       #:bound-example? [bound-example? bound-hilite?]
                       #:binding-alt? [binding-alt? bound-example?]
                       #:binding-hilite? [binding-hilite? binding-alt?]
                       #:binding-example? [binding-example? binding-hilite?])
  (define (show v?) (if v? values ghost))
  (slide
   #:title "Binding Terminology"
   (para (bit "binding") "---" "where an identifier gets its meaning")
   ((show binding-example?)
    (code {let {[#,(hilite #:on? binding-hilite? (code x) #:color binding-color) 5]} ....}))
   ((show binding-alt?)
    (code {define {f #,(hilite #:on? binding-hilite? (code x) #:color binding-color)} ....}))
   (blank)
   (para (bit "bound") "---" "refers to a binding")
   ((show bound-example?)
    (code {let {[x 5]} .... #,(hilite #:on? bound-hilite? (code x) #:color bound-color) ....}))
   ((show bound-alt?)
    (code {define {f x} .... #,(hilite #:on? bound-hilite? (code x) #:color bound-color) ....}))
   (blank)
   (para (bit "free") "---" "does not have a binding")
   ((show free-example?)
    (code {let {[x 5]} .... #,(hilite #:on? free-hilite? (code y) #:color free-color) ....}))
   ((show free-alt?)
    (code {define {f x} .... #,(hilite #:on? free-hilite? (code y) #:color free-color) ....}))))

(when condense?
  (skip-slides 9))
(unless condense?
  (binding-slide)
  (binding-slide #:binding-example? #t)
  (binding-slide #:binding-hilite? #t)
  (binding-slide #:binding-alt? #t)
  (binding-slide #:bound-example? #t)
  (binding-slide #:bound-hilite? #t)
  (binding-slide #:bound-alt? #t)
  (binding-slide #:free-example? #t)
  (binding-slide #:free-hilite? #t))
(binding-slide #:free-alt? #t)

(define (add-bind-arrow p x1 x2)
  (pin-arrow-line (/ gap-size 2)
                  p
                  x2 ct-find
                  x1 ct-find
                  #:start-angle (* pi 1/4)
                  #:start-pull 1
                  #:end-angle (* pi -1/2)
                  #:line-width 4
                  #:color bound-color))

(slide
 #:title "Free and Bound"
 'alts
 (list
  (list
   (code {let {[x 5]}
           {let {[y x]}
             {+ y {+ z x}}}}))
  (list
   (code {let {[#,(hilite (code x) #:color binding-color) 5]}
           {let {[#,(hilite (code y) #:color binding-color) #,(hilite (code x) #:color bound-color)]}
             {+ #,(hilite (code y) #:color bound-color) {+ #,(hilite (code z) #:color free-color) #,(hilite (code x) #:color bound-color)}}}}))
  (list
   (let ([mk (lambda (p)
               (code {let {[x 5]}
                       #,p}))])
     (define inner
       (code {let {[#,(hilite (code y) #:color binding-color) #,(hilite (code x) #:color free-color)]}
               {+ #,(hilite (code y) #:color bound-color) {+ #,(hilite (code z) #:color free-color) #,(hilite (code x) #:color free-color)}}}))
     (define b (cellophane (mk (ghost inner)) 0.3))
     (pin-over b inner lt-find inner)))
  (list
   (code {let {[#,(hilite (code x) #:color binding-color) 5]}
           {let {[#,(hilite (code x) #:color binding-color) #,(hilite (code x) #:color bound-color)]}
             {+ #,(hilite (code y) #:color free-color) {+ #,(hilite (code z) #:color free-color) #,(hilite (code x) #:color bound-color)}}}}))
  (list
   (let ([mk (lambda (p)
               (code {let {[x 5]}
                       #,p}))])
     (define inner
       (code {let {[#,(hilite (code x) #:color binding-color) #,(hilite (code x) #:color free-color)]}
               {+ #,(hilite (code y) #:color free-color) {+ #,(hilite (code z) #:color free-color) #,(hilite (code x) #:color bound-color)}}}))
     (define b (cellophane (mk (ghost inner)) 0.3))
     (pin-over b inner lt-find inner)))
  (list
   (let ([x1 (code x)]
         [x2 (code x)]
         [x3 (code x)]
         [x4 (code x)])
     (define p
       (code {let {[#,(hilite x1 #:color binding-color) 5]}
               {let {[#,(hilite x3 #:color binding-color) #,(hilite x2 #:color bound-color)]}
                 {+ #,(hilite (code y) #:color free-color) {+ #,(hilite (code z) #:color free-color) #,(hilite x4 #:color bound-color)}}}}))
     (define p2 (add-bind-arrow p x1 x2))
     (pin-arrow-line (/ gap-size 2)
                     p2
                     x4 cb-find
                     x3 lb-find
                     #:start-angle (* pi -1/2)
                     #:end-angle (* pi 1/4)
                     #:line-width 4
                     #:color bound-color)))
  (list
   (code {define {double x} {+ x x}}
         code:blank
         {double 3}))
  (list
   (let ([double1 (code double)]
         [double2 (code double)]
         [x1 (code x)]
         [x2 (code x)]
         [x3 (code x)])
     (define rhs (code {+ #,(hilite x2 #:color bound-color) #,(hilite x3 #:color bound-color)}))
     (let* ([p (code {define {#,(hilite double1 #:color binding-color) #,(hilite x1 #:color binding-color)} #,rhs}
                    code:blank
                    {#,(hilite double2 #:color bound-color) 3})]
            [p (add-bind-arrow p x1 x2)]
            [p (add-bind-arrow p x1 x3)]
            [p (add-bind-arrow p double1 double2)])
       p)))))
