#lang slideshow
  
(require "colors.ss" "utils.ss")

(define TypeColor GreenColor)
(define @HiliteColor RedColor)

(define in-angles
  (lambda (s)
    (format "~a~a~a" 
            (if #f #;printing? "\u2329" "\u27E8")
            s
            (if #f #;printing? "\u232A" "\u27E9"))))

(define (ct s) (tt s))
(define (idt s) (colorize (tt s) "darkblue"))
(define (vt s) (colorize (tt s) "forestgreen"))

(define alg-patterns
  (list
   (regexp "^\\^")
   (lambda (s) (ct " "))

   (regexp "^[*][*],")
   (lambda (s)  (colorize (hbl-append (t "*")
                                      (text "(,)" `(superscript . ,(current-main-font)) (current-font-size)))
                          RedColor))
   
   (regexp "^[*][*]")
   (lambda (s) (colorize (t "*") RedColor))

   (regexp "^({{|}})")
   (lambda (s) (colorize (t (string (string-ref s 0))) RedColor))

   (regexp "^[+][+]")
   (lambda (s) (colorize (text "+" `(superscript . ,(current-main-font)) (current-font-size)) RedColor))

   (regexp "^[|][-]")
   (lambda (s) (colorize turnstyle MetaColor))
   
   (regexp "^[|]")
   (lambda (s) (colorize (tt " | ") MetaColor))

   (regexp "^Omega")
   (lambda (s) (text "W" 'symbol (current-font-size)))

   (regexp "^(numTd|boolTd|intTd|voidTd|colorfishTd|fishTd)")
   (lambda (s) (ct (substring s 0 (- (string-length s) 2))))

   (regexp "^(numT|boolT|intT|fishT|objectT|colorfishT|c1T|voidT|dogT|animalT|hairstyleT|treeT)")
   (lambda (s) (colorize (tt (substring s 0 (- (string-length s) 1))) TypeColor))

   (regexp "^(number|primapp-|a-program|a-funcdef|-prim|-env|identity|prime[?])")
   (lambda (s) (ct s))

   (regexp "^#[a-zA-Z-]+#")
   (lambda (s) (ct (substring s 1 (- (string-length s) 1))))

   (regexp "^(define-type|type-case|string-append|boolean[?])")
   (lambda (s) (ct s))

   (regexp "^(prog|id|variant-id|expr|expr-or-defn|cexpr|defn|funcdef|num|bool|symbol|cond-line|val|expval|denval|lst|defined-id|list-of-num|l-o-n|list-of-sym|prim|env|reference|type|tyexpr|class-decl|method-decl|field-decl|variant|form|clause|string|[?][?][?][?])")
   (lambda (s) (colorize (t (in-angles s)) MetaColor))

   (regexp "^(LIST)")
   (lambda (s) (colorize (t (in-angles (string-downcase s))) MetaColor))

   (regexp "^Env")
   (lambda (s) (colorize (it "E") MetaColor))
   (regexp "^Type")
   (lambda (s) (colorize (it "T") MetaColor))
   (regexp "^Expr")
   (lambda (s) (colorize (it "e") MetaColor))

   (regexp "^Lambda")
   (lambda (s) (symbol (char->integer #\l)))

   (regexp "^Proc")
   (lambda (s) (colorize (t (in-angles "proc")) MetaColor))

   (regexp "^(define|and|or|'|cond|else|cons|list|void|lambda|if|local|set!)")
   (lambda (s) (ct s))

   (regexp "^#[tf]")
   (lambda (s) (vt s))
   
   (regexp "^isin[?]")
   (lambda (s) (colorize (hbl-append sym:in (ct "?")) RedColor))

   (regexp "^isin")
   (lambda (s) (colorize sym:in MetaColor))

   (regexp "^[.][.][.][.]")
   (lambda (s) (colorize (t "...") MetaColor))
   
   (regexp "^[.][.][.]")
   (lambda (s) (colorize (t "...") "darkgray"))
   
   (regexp "^[a-zA-Z.][a-zA-Z0-9.?!]*")
   (lambda (s) (idt s))
   
   (regexp "^~[a-zA-Z.][a-zA-Z0-9.?!]*")
   (lambda (s) (it (substring s 1 (string-length s))))
   
   (regexp "^,,")
   (lambda (s) (colorize (t ",") MetaColor))
   
   (regexp "^==")
   (lambda (s) (colorize (t "=") MetaColor))

   (regexp "^!=")
   (lambda (s) (colorize (t "≠") MetaColor))
   
   (regexp "^[(][*]")
   (lambda (s) (colorize (t "(") TypeColor))
   
   (regexp "^[*][)]")
   (lambda (s) (colorize (t ")") TypeColor))
   
   (regexp "^[(][[][*]")
   (lambda (s) (colorize (t "[") TypeColor))
   
   (regexp "^[*][]][)]")
   (lambda (s) (colorize (t "]") TypeColor))
   
   (regexp "^[(]~")
   (lambda (s) (t "("))
   
   (regexp "^~[)]")
   (lambda (s) (t ")"))
   
   (regexp "^[*][-]>")
   (lambda (s) (colorize sym:rightarrow TypeColor))
   
   (regexp "^[*]X[*]")
   (lambda (s) (colorize (symbol 180) TypeColor))

   (regexp "^[*]:[*]")
   (lambda (s) (colorize (tt ":") TypeColor))

   (regexp "^[-]>>")
   (lambda (s) (colorize (hbl-append sym:rightarrow sym:rightarrow) MetaColor))
   
   (regexp "^[-][-]>")
   (lambda (s) (colorize (tt " -> ") MetaColor))
   
   (regexp "^[-]>")
   (lambda (s) (colorize sym:rightarrow MetaColor))
   
   (regexp "^_>")
   (lambda (s) (tt "->"))
   
   (regexp "^__")
   (lambda (s) (tt "_"))
   
   (regexp "^<loopforever>")
   (lambda (s) (colorize (it "[loop forever]") MetaColor))
   
   (regexp "^\"[^\"]*\"")
   (lambda (s) (vt s))
   
   (regexp "^[0-9]+")
   (lambda (s) (vt s))
   
   (regexp "^_[0-9ni]+[aq]*")
   (lambda (s) (colorize (text (substring s 1 (string-length s)) `(subscript bold . ,(current-main-font)) (inexact->exact (floor (* 1.2 (current-font-size)))))
                         MetaColor))

   (regexp "^_[a-zA-Z]")
   (lambda (s) (colorize (text (substring s 1 (string-length s)) `(subscript bold italic . ,(current-main-font)) (inexact->exact (floor (* 1.2 (current-font-size)))))
                         MetaColor))

   (regexp "^@[a-zA-Z<>0-9]+")
   (lambda (s) (colorize (ct (substring s 1 (string-length s))) @HiliteColor))
   
   (regexp "^([()*,+/ =_:{}<>;#&?%]|[-]|[[]|[]])")
   (lambda (s) (ct s) #;(text s (current-main-font) (current-font-size)))

   ;; non-breaking space:
   (regexp (format "^~a" (integer->char 160)))
   (lambda (s) (text s (current-main-font) (current-font-size)))))

(define (alg-code$ p s)
  (if (string=? s "")
      (t "")
      (let ([m (and p (regexp-match-positions p s))])
        (if m
            (hbl-append
             (alg-code$ #f (substring s 0 (caar m)))
             (colorize (alg-code$ #f (substring s (caar m) (cdar m)))
                       PurpleColor)
             (alg-code$ p (substring s (cdar m) (string-length s))))
            (code/patterns alg-patterns s)))))

(define (alg-code t)
  (alg-code$ #f t))

(define (alg-code$* p . l)
  (apply
   vl-append
   (current-line-sep)
   (map (lambda (x) (if (string? x) (alg-code$ p x) x)) l)))

(define (alg-code* . l)
  (apply alg-code$* #f l))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define eqls (colorize (t "::=") MetaColor))
(define -or- (colorize (t "|") MetaColor))

(define prog-line
  (list (alg-code "prog") eqls (alg-code "defn** expr")  (blank)))
(define defn-line
  (list (alg-code "defn") eqls (alg-code "id(id) = expr")  (blank)))

(define prog+defn-lines
  (append prog-line defn-line))

(define grammar-arrow (colorize (arrow (* 3/4 (current-font-size)) (- pi)) RedColor))

(define (expr-lines a)
  (let ([arrow grammar-arrow])
    (list (alg-code "expr") eqls (alg-code "(expr + expr)") (if (= a 0) arrow (ghost arrow))
          (blank)           -or- (alg-code "(expr - expr)") (if (= a 1) arrow (blank))
          (blank)           -or- (alg-code "id(expr)") (if (= a 2) arrow (blank))
          (blank)           -or- (alg-code "id") (if (= a 3) arrow (blank))
          (blank)           -or- (alg-code "num") (if (>= a 4) arrow (blank)))))

(define id-line
  (list (alg-code "id")   eqls (hbl-append (colorize (t "a variable name: ") MetaColor)
                                           (alg-code "f,, x,, y,, z,, ")
                                           (colorize (t "...") MetaColor))  (blank)))

(define num-line
  (list (alg-code "num")    eqls (hbl-append (colorize (t "a number: ") MetaColor)
                                             (alg-code "1,, 42,, 17,, ")
                                             (colorize (t "...") MetaColor)) (blank)))

(define id+n-lines
  (append id-line num-line))

(define (repeat s)
  (hbl-append s (colorize (t "*") RedColor)))

(define (grammar-table l)
  (table
   4
   l
   (list* ltl-superimpose ctl-superimpose ltl-superimpose) ltl-superimpose
   (current-font-size) (current-line-sep)))

(define (delete-grammar-line table tag lhs rhs)
  (lt-superimpose
   table
   (inset (colorize (linewidth 3 (hline (- (pict-width table) 
                                           (pict-width lhs)
                                           (pict-width rhs))
                                        0))
                    RedColor)
          (pict-width lhs)
          (let-values ([(x y) (cc-find table tag)])
            y)
          0 0)))

(define complete-table
  (grammar-table
   (append
    prog+defn-lines
    (expr-lines -1)
    (list (blank) (blank) (blank)  (blank))
    id+n-lines)))

(define (two-line-reduction l1 l2 l2x)
  (ht-append
   (current-font-size)
   (vl-append
    (current-line-sep)
    l1
    l2)
   (alg-code " -> ")
   (vl-append
    (current-line-sep)
    l1
    l2x)))

(define (beta*-reduction val? free?)
  (let ([val (if val? "val" "expr")])
    (vc-append
     (/ (current-font-size) 2)
     (let ([top (alg-code "... (define (id_0 id_1....id_k) expr_a) ...")])
       (vc-append
        (/ (current-font-size) 2)
        (vl-append
         (current-line-sep)
         top
         (alg-code (format "... (id_0 ~a_1....~a_k) ..." val val)))
        (alg-code "->")
        (vl-append
         (current-line-sep)
         top
         (alg-code "... expr_b ..."))))
     (para #:fill? #f
           "where" (alg-code "expr_b") "is" (alg-code "expr_a") 
           "with" (if free? (background YellowColor (ct "free")) "") (alg-code "id_i") 
           "replaced by" (alg-code (format "~a_i" val))))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (steps prog . l)
  (let loop ([l l])
    (cons
     (list (two-line-reduction prog (car l) (cadr l)))
     (if (null? (cddr l))
         null
         (loop (cdr l))))))

(provide alg-patterns
         alg-code alg-code*
         alg-code$ alg-code$*
         
         eqls -or-
         prog-line defn-line prog+defn-lines
         expr-lines
         id-line num-line id+n-lines
         grammar-table delete-grammar-line repeat grammar-arrow
         complete-table
         two-line-reduction
         beta*-reduction
         steps)