(module utils (lib "slideshow.ss" "slideshow")
  (require (lib "class.ss")
           (lib "etc.ss")
           (lib "mred.ss" "mred")
           (lib "balloon.ss" "texpict")
           (lib "step.ss" "slideshow")
           (lib "symbol.ss" "texpict"))

  (provide tri-path right-tri-path left-tri-path
	   path->pict
	   add-gp-arrow
	   part-slide part-para
	   evenize
	   tts
           java
           classbox field-connect add-subclass-lines
           classbox-subtype-color classbox-arrow-color classbox-linewidth)

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define (make-tri-path x1 y1 x2 y2 x3 y3)
    (define tri-path (make-object dc-path%))
    (send tri-path move-to x1 y1)
    (send tri-path line-to x2 y2)
    (send tri-path line-to x3 y3)
    (send tri-path close)
    tri-path)
  
  (define tri-path (make-tri-path 0 1 0.5 0 1 1))
  (define h-tri-path (make-tri-path 0 0.5 1 0 1 1))
  (define right-tri-path (make-tri-path 1 1  1 0 0 1))
  (define left-tri-path (make-tri-path 0 1 0 0 1 1))
  
  (define path->pict
    (opt-lambda (path w h [filled? #t])
      (let ([p (make-object dc-path%)])
        (send p append path)
        (send p scale w h)
        (dc (lambda (dc x y)
              (let ([b (send dc get-brush)])
                (unless filled?
                  (send dc set-brush "white" 'transparent))
                (send dc draw-path p x y)
                (unless filled?
                  (send dc set-brush b))))
            w h))))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define add-gp-arrow
    (opt-lambda (p fxr fyr txr tyr [color classbox-arrow-color] [to-left? #f])
      (let* ([w (pict-width p)]
	     [h (pict-height p)]
	     [fx (* w fxr)]
	     [fy (* h fyr)]
	     [tx (* w txr)]
	     [ty (* h tyr)]
	     [cx (* w (if to-left?
			  (/ (min fxr txr) 2)
			  (- 1 (/ (- 1 (max fxr txr)) 2))))]
	     [cy (* h (/ (+ fyr tyr) 2))])
	(lt-superimpose
	 p
	 (colorize (inset (arrowhead gap-size (atan (- (- ty cy))
						    (- tx cx)))
			  (- tx (/ gap-size 2))
			  (- ty (/ gap-size 2))
			  0 0)
		   color)
	 (colorize
	  (linewidth
	   3
	   (dc (lambda (dc x y)
		 (send dc draw-spline 
		       (+ x fx) (+ y fy)
		       (+ x cx) (+ y cy)
		       (+ x tx) (+ y ty)))
	       w h 0 0))
	  color)))))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  (define (part-slide num title expl)
    (let ([part-title (if num
			  (format "Part ~a: ~a" num title)
			  title)])
      (slide/name/center
       part-title
       (ghost expl)
       (blank)
       (page-para (blank gap-size 0) (titlet part-title))
       (blank)
       expl)))
  
  (define (part-para . t)
    (page-para/r (apply para (* 1/2 client-w) t)
                 (blank (* 1/10 client-w) 0)))
  
  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define (evenize p)
    (let ([w (pict-width p)])
      (cond
        [(and (integer? w) (even? w))
         p]
        [(odd? (ceiling w))
         (inset p (- (add1 (ceiling w)) w) 0 0 0)]
        [else
         (inset p (- (ceiling w) w) 0 0 0)])))
  
  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define (tts . l) (apply vl-append line-sep (map tt l)))
     
  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define (one-java s)
    (if (regexp-match #rx"^_" s)
        (text (substring s 1) `(italic bold . modern) (current-font-size))
        (tt s)))
  
  (define (java . l)
    (apply vl-append line-sep (map one-java l)))
  
  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     
  (define (-java x)
    (inset (if (string? x)
               (java x)
               x)
           (* 4 line-sep) 0))
  
  (define classbox-color (make-object color% 120 120 120))
  (define classbox-linewidth 2)
  (define classbox-arrow-color "purple")
  (define classbox-subtype-color "blue")
  
  (define (classbox name fields methods)
    (let ([name (-java name)]
          [fields (map -java fields)]
          [methods (map -java methods)])
      (let ([w (apply max (pict-width name)
                      (append (map pict-width fields)
                              (map pict-width methods)))])
        (let ([sep (colorize (linewidth classbox-linewidth (hline w (* 2 line-sep)))
                             classbox-color)])
          (let ([name+fields (if (null? fields)
                                 name
                                 (apply vl-append
                                        line-sep
                                        name
                                        sep
                                        fields))])
            (let ([all (if (null? methods)
                           name+fields
                           (apply vl-append
                                  line-sep
                                  name+fields
                                  sep
                                  methods))])
              (color-frame all classbox-color classbox-linewidth)))))))
  
  (define (right-field-edge from-class)
    (lambda (a b)
      (let-values ([(x y) (rc-find a b)]
                   [(x0 y0) (rc-find a from-class)])
        (values x0 y))))
  
  (define (class-name-edge xtl-find)
    (lambda (a b)
      (let-values ([(x y0) (xtl-find a b)]
                   [(x1 y1) (lt-find a b)])
        (values x (/ (+ y0 y1) 2)))))
  
  (define around-connect-delta gap-size)
  
  (define (pin-xy-line p x1 y1 x2 y2 lw color)
    (pin-line p
              p (lambda (a b) (values x1 y1))
              p (lambda (a b) (values x2 y2))
              lw color))
  
  (define (field-connect p from-class from to-class)
    (let-values ([(find-from find-to)
                  (let-values ([(fx fy) (rc-find p from-class)]
                               [(tx ty) (rc-find p to-class)])
                    (cond
                      [(< ty fy)
                       ;; To right, up, back left
                       (let-values ([(x0 y0) ((right-field-edge from-class) p from)]
                                    [(x4 y4) ((class-name-edge rtl-find) p to-class)])
                         (let* ([p (pin-xy-line p 
                                                x0 y0
                                                (+ (max x0 x4) around-connect-delta) y0
                                                classbox-linewidth classbox-arrow-color)]
                                [p (pin-xy-line p 
                                                (+ (max x0 x4) around-connect-delta) y0
                                                (+ (max x0 x4) around-connect-delta) y4
                                                classbox-linewidth classbox-arrow-color)]
                                [p (pin-xy-line p 
                                                (+ (max x0 x4) around-connect-delta) y4
                                                x4 y4
                                                classbox-linewidth classbox-arrow-color)])
                           (values #f p)))]
                      [(< fx tx)
                       ;; Connection to right
                       (values (right-field-edge from-class)
                               (class-name-edge ltl-find))]
                      [else
                       ;; Connection to top
                       (values cb-find ct-find)]))])
      (if find-from
          (pin-line p 
                    from find-from
                    to-class find-to
                    classbox-linewidth classbox-arrow-color)
          ;; hack: find-to is the result pict
          find-to)))
  
  (define sub-tri-height (/ gap-size 2))
  
  (define add-subclass-lines
    (opt-lambda (p super subs [horiz? #f])
      (define _ct-find (if horiz? 
                           (lambda (p a)
                             (let-values ([(x y) (lc-find p a)])
                               (values y x)))
                           ct-find))
      (define _lt-find (if horiz? 
                           (lambda (p a)
                             (let-values ([(x y) (lt-find p a)])
                               (values y x)))
                           ct-find))
      (define _cb-find (if horiz?
                           (lambda (p a)
                             (let-values ([(x y) (rc-find p a)])
                               (values y x)))
                           cb-find))
      (define _vl-append (if horiz?
                             ht-append
                             vl-append))
      (define _vc-append (if horiz?
                             hc-append
                             vc-append))
      (define _vline (if horiz?
                         (lambda (w h) (hline h w))
                         vline))
      (define _hline (if horiz?
                         (lambda (w h) (vline h w))
                         hline))
      (define _inset (if horiz?
                         (lambda (p l t r b) (inset p t l b r))
                         inset))
      (define _pict-width (if horiz?
                              pict-height
                              pict-width))
      (let-values ([(x y) (_cb-find p super)])
        (define _ct-find-one (if ((length subs) . > . 1)
                                 ct-find
                                 (lambda (p a)
                                   (let-values ([(ax ay) (_lt-find p a)])
                                     (if (<= ax x (+ ax (_pict-width a)))
                                         (values x ay)
                                         (_ct-find p a))))))
        (let ([xs (map (lambda (sub)
                         (let-values ([(x y) (_ct-find-one p sub)])
                           x))
                       subs)]
              [ys (map (lambda (sub)
                         (let-values ([(x y) (_ct-find-one p sub)])
                           y))
                       subs)])
          (let ([min-x (apply min x xs)]
                [max-x (apply max x xs)]
                [min-y (apply min ys)])
            (let ([line-y (/ (+ min-y y) 2)])
              (place-over
               p
               (if horiz? y min-x) (if horiz? min-x y)
               (colorize
                (linewidth
                 classbox-linewidth
                 (_vl-append
                  (_inset (_vc-append (_vline 0 (- line-y y sub-tri-height))
                                      (path->pict (if horiz? h-tri-path tri-path)
                                                  (if horiz? 
                                                      sub-tri-height
                                                      (* 2 sub-tri-height))
                                                  (if horiz?
                                                      (* 2 sub-tri-height)
                                                      sub-tri-height)
                                                  #f))
                          (- x min-x sub-tri-height) 0 0 0)
                  (_hline (- max-x min-x) 0)
                  (apply lt-superimpose
                         (map (lambda (sub)
                                (let-values ([(x y) (_ct-find-one p sub)])
                                  (_inset (_vline 0 (- y line-y))
                                          (- x min-x) 0 0 0)))
                              subs))))
                classbox-subtype-color))))))))
     
  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  )
